@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: BasicSession.java 1353 2004-10-17 01:03:03Z nfiedler $
+ * $Id: BasicSession.java 1529 2005-02-21 02:19:01Z nfiedler $
  *
  ********************************************************************/
 
@@ -172,8 +172,11 @@ public class BasicSession extends AbstractSession implements VMEventListener {
             // the JDI version is 1.4 or higher.
             VirtualMachine vm = connection.getVM();
             if (vm.canGetSourceDebugExtension()) {
-                String stratum = preferences().get("defaultStratum", Defaults.STRATUM);
-                connection.getVM().setDefaultStratum(stratum);
+                String stratum = preferences().get(
+                        "defaultStratum", Defaults.STRATUM);
+                if (stratum != null && stratum.length() > 0) {
+                    connection.getVM().setDefaultStratum(stratum);
+                }
             }
 
             if (showSource) {
