@@ -27,7 +27,7 @@
  *      ----    ----            -----------
  *      nf      04/17/02        Initial version
  *
- * $Id: Defaults.java 1305 2004-09-22 07:02:24Z nfiedler $
+ * $Id: Defaults.java 1529 2005-02-21 02:19:01Z nfiedler $
  *
  ********************************************************************/
 
@@ -104,8 +104,9 @@ public class Defaults {
     public static final Hashtable KEYBOARD_SHORTS;
     /** Table of view colors, keyed by the color name. */
     public static final Hashtable VIEW_COLORS;
-    /** The default debugging stratum. */
-    public static final String STRATUM = "Java";
+    /** The default debugging stratum (leave blank so the classes can
+     * provide the stratum for us). */
+    public static final String STRATUM = "";
 
     static {
         // Initialize the shortcuts table.
