@@ -1,6 +1,6 @@
 /*********************************************************************
  *
- *      Copyright (C) 2001-2004 Nathan Fiedler
+ *      Copyright (C) 2001-2005 Nathan Fiedler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: PathManager.java 1173 2004-02-27 20:51:07Z nfiedler $
+ * $Id: PathManager.java 1785 2005-07-07 18:48:50Z nfiedler $
  *
  ********************************************************************/
 
@@ -34,7 +34,6 @@ import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.util.Enumeration;
-import java.util.Hashtable;
 import java.util.Iterator;
 import java.util.List;
 import java.util.StringTokenizer;
@@ -43,7 +42,6 @@ import java.util.zip.ZipFile;
 import org.gjt.jclasslib.io.ClassFileReader;
 import org.gjt.jclasslib.structures.AttributeInfo;
 import org.gjt.jclasslib.structures.ClassFile;
-import org.gjt.jclasslib.structures.CPInfo;
 import org.gjt.jclasslib.structures.InvalidByteCodeException;
 import org.gjt.jclasslib.structures.attributes.SourceFileAttribute;
 import org.gjt.jclasslib.structures.constants.ConstantUtf8Info;
@@ -434,6 +432,14 @@ public class PathManager implements Manager {
             String classname = location.declaringType().name();
             String filename = ClassName.toFilename(classname, source);
             SourceSource src = mapFile(filename);
+            if (src == null) {
+                // Common case failed, try harder to find the source.
+                VirtualMachine vm = location.virtualMachine();
+                if (vm.canGetSourceDebugExtension()) {
+                    String path = location.sourcePath(null);
+                    src = mapFile(path);
+                }
+            }
             if (src != null) {
                 return src;
             }
@@ -469,7 +475,26 @@ public class PathManager implements Manager {
 
         SourceSource src = mapFile(filename);
         if (src == null) {
-            src = sourceFactory.create(clazz);
+            // Common case failed, try harder to find the source.
+            VirtualMachine vm = clazz.virtualMachine();
+            if (vm.canGetSourceDebugExtension()) {
+                try {
+                    List paths = clazz.sourcePaths(null);
+                    Iterator iter = paths.iterator();
+                    while (iter.hasNext()) {
+                        String path = (String) iter.next();
+                        src = mapFile(path);
+                        if (src != null) {
+                            break;
+                        }
+                    }
+                } catch (AbsentInformationException aie) {
+                    // Fall through
+                }
+            }
+            if (src == null) {
+                src = sourceFactory.create(clazz);
+            }
         }
         return src;
     } // mapSource
@@ -524,6 +549,7 @@ public class PathManager implements Manager {
         StringBuffer sb = new StringBuffer(80);
         for (int ii = 0; ii < size; ii++) {
             String entry = st.nextToken();
+            entry = entry.trim();
             File f = new File(entry);
             try {
                 sb.append(f.getCanonicalPath());
@@ -553,6 +579,7 @@ public class PathManager implements Manager {
         StringBuffer sb = new StringBuffer(80);
         for (int ii = 0; ii < size; ii++) {
             String entry = st.nextToken();
+            entry = entry.trim();
             File f = new File(entry);
             try {
                 sb.append(f.getCanonicalPath());
