@@ -27,13 +27,14 @@
  *      ----    ----            -----------
  *      MvD     03/10/02        Initial version
  *
- * $Id: JSwatFileFilter.java 1048 2003-11-16 07:27:16Z nfiedler $
+ * $Id: JSwatFileFilter.java 1541 2005-02-23 08:18:47Z nfiedler $
  *
  ********************************************************************/
 
 package com.bluemarsh.jswat.action;
 
 import java.io.File;
+import java.util.List;
 import javax.swing.filechooser.FileFilter;
 
 /**
@@ -45,8 +46,8 @@ import javax.swing.filechooser.FileFilter;
  * @author  Marko van Dooren
  */
 public class JSwatFileFilter extends FileFilter {
-    /** The extension of this JSwatFileFilter. */
-    private String extension;
+    /** The extensions of this JSwatFileFilter. */
+    private List extensions;
     /** The description of this JSwatFileFilter. */
     private String description;
 
@@ -54,12 +55,12 @@ public class JSwatFileFilter extends FileFilter {
      * Initialize a new JSwatFileFilter with the given extension
      * and description.
      *
-     * @param extension    the extension of the new JSwatFileFilter.
+     * @param extensions   the extensions of the new JSwatFileFilter.
      * @param description  description for the file type of the new
      *                     JSwatFileFilter.
      */
-    public JSwatFileFilter(String extension, String description) {
-        this.extension = extension.toLowerCase();
+    public JSwatFileFilter(List extensions, String description) {
+        this.extensions = extensions;
         this.description = description;
     } // JSwatFileFilter
 
@@ -71,9 +72,19 @@ public class JSwatFileFilter extends FileFilter {
      * @return  true if the pathname should be included.
      */
     public boolean accept(File pathname) {
-        return (pathname != null)
-            && ((pathname.isDirectory())
-                || (pathname.getName().toLowerCase().endsWith(extension)));
+        if (pathname == null)
+            return false;
+
+        if (pathname.isDirectory())
+            return true;
+
+        String name = pathname.getName().toLowerCase();
+
+        for (java.util.Iterator i = extensions.iterator(); i.hasNext();)
+            if (name.endsWith((String) i.next()))
+                return true;
+
+        return false;
     } // accept
 
     /**
@@ -90,7 +101,7 @@ public class JSwatFileFilter extends FileFilter {
      *
      * @return  file name extension.
      */
-    public String getExtension() {
-        return extension;
+    public List getExtensions() {
+        return extensions;
     } // getExtension
 } // JSwatFileFilter
