@@ -33,7 +33,7 @@
  *      nf      03/18/02        Fixed bug 89
  *      nf      12/23/02        Fixed bug 653
  *
- * $Id: OpenFileAction.java 1059 2003-11-22 22:20:25Z nfiedler $
+ * $Id: OpenFileAction.java 1541 2005-02-23 08:18:47Z nfiedler $
  *
  ********************************************************************/
 
@@ -86,9 +86,11 @@ public class OpenFileAction extends JSwatAction {
         JFileChooser fc = new JFileChooser(lastDir);
         fc.addChoosableFileFilter(fc.getAcceptAllFileFilter());
         fc.setFileHidingEnabled(false);
-        String ext = Preferences.userRoot().node(
-            "com/bluemarsh/jswat").get(
-                "defaultFileExtension", Defaults.FILE_EXTENSION);
+        // Hard-coded list of known source file extensions.
+        java.util.List ext = new java.util.LinkedList();
+        ext.add(".java");
+        ext.add(".jsp");
+        ext.add(".nice");
         fc.setFileFilter(new JSwatFileFilter(ext, "Source Files"));
         int response = fc.showOpenDialog(win);
 
