@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: PreferencesAction.java 1113 2004-01-25 22:25:32Z nfiedler $
+ * $Id: PreferencesAction.java 1305 2004-09-22 07:02:24Z nfiedler $
  *
  ********************************************************************/
 
@@ -429,6 +429,8 @@ class PreferencesDialog extends JDialog implements ActionListener {
     protected class MiscPanel extends JPanel {
         /** Field for entering the default file extension. */
         private JTextField fileExtField;
+        /** Field for entering the default source stratum. */
+        private JTextField stratumField;
         /** Spinner for specifying the mro list size. */
         private JSpinner mroListSpinner;
         /** Spinner for specifying the invocation timeout. */
@@ -470,6 +472,10 @@ class PreferencesDialog extends JDialog implements ActionListener {
                 "com/bluemarsh/jswat");
             prefs.put("defaultFileExtension", fileExtField.getText());
 
+            prefs = Preferences.userRoot().node(
+                "com/bluemarsh/jswat/Session");
+            prefs.put("defaultStratum", stratumField.getText());
+
             prefs = Preferences.userRoot().node(
                 "com/bluemarsh/jswat/ui/mro");
             Integer max = (Integer) mroListSpinner.getValue();
@@ -549,6 +555,23 @@ class PreferencesDialog extends JDialog implements ActionListener {
             gbl.setConstraints(fileExtField, gbc);
             panel.add(fileExtField);
 
+            label = new JLabel(Bundle.getString(
+                "Preferences.misc.stratum"));
+            gbc.anchor = GridBagConstraints.EAST;
+            gbc.gridwidth = GridBagConstraints.RELATIVE;
+            gbl.setConstraints(label, gbc);
+            panel.add(label);
+
+            prefs = Preferences.userRoot().node(
+                "com/bluemarsh/jswat/Session");
+            val = prefs.get("defaultStratum", Defaults.STRATUM);
+            stratumField = new JTextField(val, 10);
+            gbc.anchor = GridBagConstraints.WEST;
+            gbc.gridwidth = GridBagConstraints.REMAINDER;
+            gbc.weightx = 1.0;
+            gbl.setConstraints(stratumField, gbc);
+            panel.add(stratumField);
+
             return panel;
         } // createMapperPanel
 
