@@ -1,6 +1,6 @@
 /*********************************************************************
  *
- *      Copyright (C) 2001-2003 Nathan Fiedler
+ *      Copyright (C) 2001-2004 Nathan Fiedler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,19 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * PROJECT:     JSwat
- * MODULE:      Breakpoints
- * FILE:        BreakpointGroup.java
- *
- * AUTHOR:      Nathan Fiedler
- *
- * REVISION HISTORY:
- *      Name    Date            Description
- *      ----    ----            -----------
- *      nf      03/24/01        Initial version
- *      nf      06/11/01        Added group and breakpoint iterators
- *
- * $Id: BreakpointGroup.java 1146 2004-02-20 03:41:14Z nfiedler $
+ * $Id: BreakpointGroup.java 1313 2004-09-29 05:20:24Z nfiedler $
  *
  ********************************************************************/
 
@@ -55,6 +43,8 @@ public class BreakpointGroup {
     protected String groupName;
     /** True if this breakpoint group is enabled. */
     protected boolean isEnabled;
+    /** True if this breakpoint group is expanded in the breakpoints panel. */
+    protected boolean isExpanded;
     /** List of breakpoint groups in this group. */
     protected List groupList;
     /** List of breakpoints in this group. */
@@ -264,6 +254,15 @@ public class BreakpointGroup {
         }
     } // isEnabled
 
+    /**
+     * Returns true if this group is expanded.
+     *
+     * @return  true if the group has been expanded visually, false otherwise.
+     */
+    public boolean isExpanded() {
+        return isExpanded;
+    } // isExpanded
+
     /**
      * Reads the breakpoint properties from the given preferences node.
      *
@@ -274,6 +273,7 @@ public class BreakpointGroup {
     public boolean readObject(Preferences prefs) {
         isEnabled = prefs.getBoolean("isEnabled", true);
         groupName = prefs.get("groupName", "(undefined)");
+        isExpanded = prefs.getBoolean("isExpanded", true);
 
         // Look for persisted breakpoints.
         Session session = getSession();
@@ -415,6 +415,15 @@ public class BreakpointGroup {
         isEnabled = enabled;
     } // setEnabled
 
+    /**
+     * Sets the expanded state of the group.
+     *
+     * @param  expanded  new expanded state.
+     */
+    public void setExpanded(boolean expanded) {
+        isExpanded = expanded;
+    } // setExpanded
+
     /**
      * Changes the name of this breakpoint group to that which is given.
      *
@@ -453,6 +462,7 @@ public class BreakpointGroup {
      */
     public boolean writeObject(Preferences prefs) {
         prefs.putBoolean("isEnabled", isEnabled);
+        prefs.putBoolean("isExpanded", isExpanded);
         prefs.put("groupName", groupName);
 
         // For each breakpoint, make a new preference node and
