@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: MethodBreakpoint.java 1245 2004-04-30 02:33:53Z nfiedler $
+ * $Id: MethodBreakpoint.java 1456 2005-01-26 09:18:50Z nfiedler $
  *
  ********************************************************************/
 
@@ -72,7 +72,7 @@ public class MethodBreakpoint extends LocationBreakpoint {
      * @throws  MalformedMemberNameException
      *          if the method name is invalid.
      */
-    MethodBreakpoint(String classPattern, String methodId, List methodArgs)
+    public MethodBreakpoint(String classPattern, String methodId, List methodArgs)
         throws ClassNotFoundException,
                MalformedMemberNameException {
 
