@@ -1,6 +1,6 @@
 /*********************************************************************
  *
- *      Copyright (C) 2001-2003 Nathan Fiedler
+ *      Copyright (C) 2001-2005 Nathan Fiedler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: ResolvableBreakpoint.java 1081 2003-12-03 07:47:18Z nfiedler $
+ * $Id: ResolvableBreakpoint.java 1511 2005-02-13 07:59:54Z nfiedler $
  *
  ********************************************************************/
 
@@ -34,7 +34,6 @@ import com.sun.jdi.request.InvalidRequestStateException;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
-import java.util.ListIterator;
 import java.util.StringTokenizer;
 import java.util.logging.Level;
 import java.util.prefs.Preferences;
@@ -50,15 +49,16 @@ import java.util.prefs.Preferences;
 public abstract class ResolvableBreakpoint extends AbstractBreakpoint {
     /** Specification for the class this breakpoint is meant for. */
     protected ReferenceSpec referenceSpec;
-    /** Class prepare request used to resolve the breakpoint. */
-    protected ClassPrepareRequest prepareRequest;
+    /** List of ClassPrepareRequest used to resolve the breakpoint. */
+    protected transient List prepareRequests;
     /** Resolved event request if any, or null if not yet resolved. */
-    protected EventRequest eventRequest;
+    protected transient EventRequest eventRequest;
 
     /**
      * Default constructor for deserialization.
      */
     ResolvableBreakpoint() {
+        prepareRequests = new ArrayList();
     } // ResolvableBreakpoint
 
     /**
@@ -70,9 +70,22 @@ public abstract class ResolvableBreakpoint extends AbstractBreakpoint {
      *          if classId is not a valid identifier.
      */
     ResolvableBreakpoint(String classId) throws ClassNotFoundException {
+        this();
         referenceSpec = createReferenceSpec(classId);
     } // ResolvableBreakpoint
 
+    /**
+     * Adds a class prepare request to by managed by this breakpoint.
+     *
+     * @param  req  class prepare request to associate with this breakpoint.
+     */
+    protected void addPrepareRequest(ClassPrepareRequest req) {
+        prepareRequests.add(req);
+        // Save reference so we can determine ownership later.
+        req.putProperty("breakpoint", this);
+        req.enable();
+    }
+
     /**
      * Constructs a ReferenceSpec of the appropriate type.
      *
@@ -109,26 +122,30 @@ public abstract class ResolvableBreakpoint extends AbstractBreakpoint {
     /**
      * Delete the class prepare request so we can resolve all over.
      */
-    protected void deletePrepareRequest() {
+    protected void deletePrepareRequests() {
         // Don't want to know about the class prepare events.
-        if (prepareRequest != null) {
+        if (!prepareRequests.isEmpty()) {
+            Iterator iter = prepareRequests.iterator();
             try {
-                VirtualMachine vm = prepareRequest.virtualMachine();
-                EventRequestManager erm = vm.eventRequestManager();
-                erm.deleteEventRequest(prepareRequest);
+                while (iter.hasNext()) {
+                    EventRequest er = (EventRequest) iter.next();
+                    VirtualMachine vm = er.virtualMachine();
+                    EventRequestManager erm = vm.eventRequestManager();
+                    erm.deleteEventRequest(er);
+                }
             } catch (VMDisconnectedException vmde) {
                 // this will happen all the time
             }
-            prepareRequest = null;
+            prepareRequests.clear();
         }
-    } // deletePrepareRequest
+    } // deletePrepareRequests
 
     /**
      * Tear down this breakpoint in preparation for deletion.
      */
     public void destroy() {
         deleteEventRequest();
-        deletePrepareRequest();
+        deletePrepareRequests();
         super.destroy();
     } // destroy
 
@@ -183,7 +200,7 @@ public abstract class ResolvableBreakpoint extends AbstractBreakpoint {
     public void reset() {
         // Make this breakpoint unresolved so it will be resolved again.
         deleteEventRequest();
-        deletePrepareRequest();
+        deletePrepareRequests();
         // Now do the usual reset thing, including notifying listeners.
         super.reset();
     } // reset
@@ -192,26 +209,26 @@ public abstract class ResolvableBreakpoint extends AbstractBreakpoint {
      * Try to resolve this breakpoint against the class prepare event.
      *
      * @param  event  class prepare event
-     * @return  event request, or null if it could not be resolved.
+     * @return  true if resolved, false otherwise.
      * @throws  ResolveException
      *          if the resolution fails in a bad way.
      */
-    public EventRequest resolveAgainstEvent(ClassPrepareEvent event)
+    public boolean resolveAgainstEvent(ClassPrepareEvent event)
         throws ResolveException {
         ReferenceType clazz = event.referenceType();
-        // Third conditional is for NJPL support.
-        if (prepareRequest != null && prepareRequest.equals(event.request())
-            && referenceSpec.matches(clazz)) {
+        EventRequest er = null;
+        if (referenceSpec.matches(clazz)) {
             // It looks like a match, let's try to resolve the request.
-            EventRequest er = resolveReference(clazz);
+            er = resolveReference(clazz);
             if (er != null) {
                 // Remove the previous event request, if any, but only
                 // if we successfully created a new breakpoint request.
                 deleteEventRequest();
                 eventRequest = er;
+                return true;
             }
         }
-        return eventRequest;
+        return false;
     } // resolveAgainstEvent
 
     /**
@@ -220,19 +237,16 @@ public abstract class ResolvableBreakpoint extends AbstractBreakpoint {
      * prepare request so we can resolve when the class is loaded.
      *
      * @param  vm  VirtualMachine
-     * @return  event request, or null if it could not be resolved.
      * @throws  ResolveException
      *          if the resolution fails in a bad way.
      */
-    public EventRequest resolveEagerly(VirtualMachine vm)
+    public void resolveEagerly(VirtualMachine vm)
         throws ResolveException {
-        if (prepareRequest == null) {
-            // Create class prepare request for this spec.
-            prepareRequest = referenceSpec.createPrepareRequest(vm);
-            // Save reference so we can determine ownership later.
-            prepareRequest.putProperty("breakpoint", this);
-            prepareRequest.enable();
-        }
+
+        // Clear out the old prepare requests to make room for the new ones.
+        deletePrepareRequests();
+        // Create class prepare requests for this spec.
+        referenceSpec.createPrepareRequest(this, vm);
 
         // Get the appropriate list of classes.
         List classes = null;
@@ -243,8 +257,6 @@ public abstract class ResolvableBreakpoint extends AbstractBreakpoint {
             classes = vm.allClasses();
             vm.resume();
         }
-        // Have to create a modifiable version of the list.
-        classes = new ArrayList(classes);
 
         // We handle exceptions specially because a class may be loaded
         // by more than one class loader, and one of those instances may
@@ -255,7 +267,7 @@ public abstract class ResolvableBreakpoint extends AbstractBreakpoint {
         EventRequest er = null;
 
         // Run through the list of classes trying to find a match.
-        ListIterator iter = classes.listIterator();
+        Iterator iter = classes.iterator();
         while (iter.hasNext()) {
             ReferenceType clazz = (ReferenceType) iter.next();
             try {
@@ -285,11 +297,6 @@ public abstract class ResolvableBreakpoint extends AbstractBreakpoint {
                     throw re;
                 }
             }
-            // Do nested classes, too.
-            Iterator nestedIter = clazz.nestedTypes().iterator();
-            while (nestedIter.hasNext()) {
-                iter.add(nestedIter.next());
-            }
         }
 
         if (er == null && originalExc != null) {
@@ -297,7 +304,6 @@ public abstract class ResolvableBreakpoint extends AbstractBreakpoint {
             throw originalExc;
         }
         eventRequest = er;
-        return eventRequest;
     } // resolveEagerly
 
     /**
@@ -331,10 +337,10 @@ public abstract class ResolvableBreakpoint extends AbstractBreakpoint {
             } catch (InvalidRequestStateException irse) {
                 // This seems to happen when the code has been redefined.
                 eventRequest = null;
-                prepareRequest = null;
+                prepareRequests.clear();
             } catch (VMDisconnectedException vmde) {
                 eventRequest = null;
-                prepareRequest = null;
+                prepareRequests.clear();
             }
         }
     } // setEnabled
@@ -379,9 +385,9 @@ public abstract class ResolvableBreakpoint extends AbstractBreakpoint {
      */
     public class ReferenceSpec {
         /** Name of the class specified, without wildcards. */
-        private String className;
+        protected String className;
         /** Name of the class specified, possibly with wild cards. */
-        private String classPattern;
+        protected String classPattern;
         /** True if class specification is suffixed with a wildcard. */
         private boolean postWild;
         /** True if class specification is prefixed with a wildcard. */
@@ -438,18 +444,33 @@ public abstract class ResolvableBreakpoint extends AbstractBreakpoint {
         } // ReferenceSpec
 
         /**
-         * Create a class prepare request appropriate for this reference
+         * Create class prepare requests appropriate for this reference
          * type specification.
          *
-         * @param  vm  VirtualMachine to use for creating request.
-         * @return  ClassPrepareRequest.
+         * @param  bp  breakpoint for which to create prepare requests.
+         * @param  vm  VirtualMachine to use for creating requests.
          */
-        public ClassPrepareRequest createPrepareRequest(VirtualMachine vm) {
-            ClassPrepareRequest request =
-                vm.eventRequestManager().createClassPrepareRequest();
-            request.addClassFilter(classPattern);
-            request.addCountFilter(1);
-            return request;
+        public void createPrepareRequest(
+                ResolvableBreakpoint bp, VirtualMachine vm) {
+            String basename = className;
+            int idx = basename.indexOf('$');
+            if (idx > -1) {
+                basename = basename.substring(0, idx);
+            }
+            // Create patterns for the base class and its inner classes.
+            String[] names = {
+                basename,
+                basename + ".*",
+                basename + "$*"
+            };
+            // Create a prepare request for each pattern.
+            for (int ii = 0; ii < names.length; ii++) {
+                ClassPrepareRequest request =
+                    vm.eventRequestManager().createClassPrepareRequest();
+                request.addClassFilter(names[ii]);
+                //request.addCountFilter(1);
+                bp.addPrepareRequest(request);
+            }
         } // createPrepareRequest
 
         /**
