@@ -1,6 +1,6 @@
 /*********************************************************************
  *
- *      Copyright (C) 2001 Nathan Fiedler
+ *      Copyright (C) 2001-2003 Nathan Fiedler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,11 +26,9 @@
  *      Name    Date            Description
  *      ----    ----            -----------
  *      nf      07/25/01        Initial version
+ *      nf      03/06/03        Fixed bug 726
  *
- * DESCRIPTION:
- *      Defines the basic breakpoint ui adapter.
- *
- * $Id: BasicBreakpointUI.java 629 2002-10-26 23:03:26Z nfiedler $
+ * $Id: BasicBreakpointUI.java 885 2003-03-09 21:54:27Z nfiedler $
  *
  ********************************************************************/
 
@@ -38,8 +36,7 @@ package com.bluemarsh.jswat.breakpoint.ui;
 
 import com.bluemarsh.jswat.breakpoint.Breakpoint;
 import com.bluemarsh.jswat.breakpoint.BreakpointGroup;
-import com.bluemarsh.jswat.breakpoint.ResolvableBreakpoint;
-import com.bluemarsh.jswat.breakpoint.ui.BreakpointUI;
+import com.bluemarsh.jswat.breakpoint.LocatableBreakpoint;
 import com.sun.jdi.request.EventRequest;
 import java.awt.Component;
 import java.awt.GridBagConstraints;
@@ -48,8 +45,19 @@ import java.awt.Insets;
 import java.util.Iterator;
 import java.util.Stack;
 import java.util.Vector;
-import javax.swing.*;
-import javax.swing.text.*;
+import javax.swing.Box;
+import javax.swing.BoxLayout;
+import javax.swing.ButtonGroup;
+import javax.swing.ButtonModel;
+import javax.swing.JCheckBox;
+import javax.swing.JComboBox;
+import javax.swing.JLabel;
+import javax.swing.JPanel;
+import javax.swing.JRadioButton;
+import javax.swing.JTextField;
+import javax.swing.text.AttributeSet;
+import javax.swing.text.BadLocationException;
+import javax.swing.text.PlainDocument;
 
 /**
  * Class BasicBreakpointUI is an adapter for building the user interface
@@ -68,33 +76,33 @@ public class BasicBreakpointUI implements BreakpointUI {
     /** Contraints object for the panel layout. */
     protected GridBagConstraints gbc;
     /** Original enabled value. */
-    protected boolean originalEnabled;
+    private boolean originalEnabled;
     /** Checkbox for the breakpoint's enabledness. */
-    protected JCheckBox enabledCheckbox;
+    private JCheckBox enabledCheckbox;
     /** Original skip count value. */
-    protected int originalSkipcount;
+    private int originalSkipcount;
     /** Breakpoint's skip count text field. */
-    protected JTextField skipcountTextfield;
+    private JTextField skipcountTextfield;
     /** Original expire count value. */
-    protected int originalExpirecount;
+    private int originalExpirecount;
     /** Breakpoint's expire count text field. */
-    protected JTextField expirecountTextfield;
+    private JTextField expirecountTextfield;
     /** Original group the breakpoint belonged to. */
-    protected BreakpointGroup originalGroup;
+    private BreakpointGroup originalGroup;
     /** Combo box showing the available breakpoint groups. */
-    protected JComboBox groupCombo;
+    private JComboBox groupCombo;
     /** Original suspend policy value. */
-    protected int originalSuspendPolicy;
+    private int originalSuspendPolicy;
     /** Radio button group for the suspend policy. */
-    protected ButtonGroup suspendGroup;
+    private ButtonGroup suspendGroup;
     /** Original list of class filters. */
-    protected String originalClassFilters;
+    private String originalClassFilters;
     /** Breakpoint's class filters text field. */
-    protected JTextField classFiltersTextfield;
+    private JTextField classFiltersTextfield;
     /** Original list of thread filters. */
-    protected String originalThreadFilters;
+    private String originalThreadFilters;
     /** Breakpoint's thread filters text field. */
-    protected JTextField threadFiltersTextfield;
+    private JTextField threadFiltersTextfield;
 
     /**
      * Create a BasicBreakpointUI that will operate on the given breakpoint.
@@ -119,10 +127,17 @@ public class BasicBreakpointUI implements BreakpointUI {
         gbl.setConstraints(enabledCheckbox, gbc);
         propertiesPanel.add(enabledCheckbox);
 
+        // Inidicate the resolution status of the breakpoint.
+        JLabel label = new JLabel(bp.isResolved()
+            ? Bundle.getString("resolvedLabel")
+            : Bundle.getString("unresolvedLabel"));
+        gbl.setConstraints(label, gbc);
+        propertiesPanel.add(label);
+
         // Allow changing the thread suspend policy.
         originalSuspendPolicy = bp.getSuspendPolicy();
         Box suspendBox = new Box(BoxLayout.X_AXIS);
-        JLabel label = new JLabel(Bundle.getString("suspendPolicyLabel"));
+        label = new JLabel(Bundle.getString("suspendPolicyLabel"));
         suspendBox.add(label);
         suspendBox.add(Box.createHorizontalStrut(10));
         gbl.setConstraints(suspendBox, gbc);
@@ -157,18 +172,17 @@ public class BasicBreakpointUI implements BreakpointUI {
         }
         suspendBox.add(radioButton);
 
-        if (bp instanceof ResolvableBreakpoint) {
-            // If the breakpoint is resolvable, it has a class name.
-            ResolvableBreakpoint rbp = (ResolvableBreakpoint) bp;
+        if (bp instanceof LocatableBreakpoint) {
+            // If the breakpoint is locatable, it may have a class name.
+            LocatableBreakpoint lbp = (LocatableBreakpoint) bp;
             label = new JLabel(Bundle.getString("classidLabel"));
             gbc.anchor = GridBagConstraints.EAST;
             gbc.gridwidth = GridBagConstraints.RELATIVE;
             gbl.setConstraints(label, gbc);
             propertiesPanel.add(label);
 
-            JTextField tf = new JTextField(
-                rbp.getReferenceTypeSpec().getIdentifier(), 20);
-            tf.setEnabled(false);
+            JTextField tf = new JTextField(lbp.getClassName(), 20);
+            tf.setEditable(false);
             gbc.anchor = GridBagConstraints.WEST;
             gbc.gridwidth = GridBagConstraints.REMAINDER;
             gbl.setConstraints(tf, gbc);
@@ -378,7 +392,7 @@ public class BasicBreakpointUI implements BreakpointUI {
         BreakpointGroup parent = bp.getBreakpointGroup();
         for (int i = 0; i < count; i++) {
             GroupNamePair pair = (GroupNamePair) box.getItemAt(i);
-            if (pair.breakpointGroup == parent) {
+            if (pair.getGroup() == parent) {
                 box.setSelectedIndex(i);
                 break;
             }
@@ -397,7 +411,7 @@ public class BasicBreakpointUI implements BreakpointUI {
         try {
             GroupNamePair grpname = (GroupNamePair) box.getSelectedItem();
             if (grpname != null) {
-                return grpname.breakpointGroup;
+                return grpname.getGroup();
             } else {
                 return null;
             }
@@ -454,8 +468,10 @@ public class BasicBreakpointUI implements BreakpointUI {
          * @param  offs  offset in which to insert.
          * @param  str   string to insert.
          * @param  a     attribute set.
+         * @throws  BadLocationException
+         *          if insertion offset is illegal.
          */
-        public void insertString(int offs, String str, AttributeSet a) 
+        public void insertString(int offs, String str, AttributeSet a)
             throws BadLocationException {
 
             if (str == null) {
@@ -483,9 +499,9 @@ public class BasicBreakpointUI implements BreakpointUI {
      */
     protected static class GroupNamePair {
         /** Breakpoint group. */
-        public BreakpointGroup breakpointGroup;
+        private BreakpointGroup breakpointGroup;
         /** Group's composite name. */
-        public String compositeName;
+        private String compositeName;
 
         /**
          * Constructs a GroupNamePair object.
@@ -498,6 +514,15 @@ public class BasicBreakpointUI implements BreakpointUI {
             compositeName = name;
         } // GroupNamePair
 
+        /**
+         * Returns the breakpoint group.
+         *
+         * @return  breakpoint group.
+         */
+        public BreakpointGroup getGroup() {
+            return breakpointGroup;
+        } // getGroup
+
         /**
          * Returns the string representation of this object.
          *
