@@ -1,6 +1,6 @@
 /*********************************************************************
  *
- *      Copyright (C) 1999-2002 Nathan Fiedler
+ *      Copyright (C) 1999-2005 Nathan Fiedler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,20 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * PROJECT:     JSwat
- * MODULE:      JSwat Commands
- * FILE:        classCommand.java
- *
- * AUTHOR:      Nathan Fiedler
- *
- * REVISION HISTORY:
- *      Name    Date            Description
- *      ----    ----            -----------
- *      nf      05/30/99        Initial version
- *      nf      07/13/02        Implemented RFE 430
- *      nf      07/26/02        Fixed bug 595
- *
- * $Id: classCommand.java 757 2002-12-24 04:10:29Z nfiedler $
+ * $Id: classCommand.java 1768 2005-07-03 23:18:52Z nfiedler $
  *
  ********************************************************************/
 
@@ -97,7 +84,7 @@ public class classCommand extends JSwatCommand {
             Matcher matcher = patt.matcher(clazz.name());
             if (matcher.find()) {
                 found = true;
-                // For each matching class, print its methods.
+                // For each matching class, print its information.
                 printClass(clazz, showAll, buf);
                 if (iter.hasNext()) {
                     buf.append('\n');
@@ -108,7 +95,7 @@ public class classCommand extends JSwatCommand {
             throw new CommandException(Bundle.getString("class.nomatch"));
         }
         out.write(buf.toString());
-    } // perform
+    }
 
     /**
      * Print information about a class, interface, or array.
@@ -215,5 +202,5 @@ public class classCommand extends JSwatCommand {
             buf.append(clr.uniqueID());
             buf.append(")\n");
         }
-    } // printClass
-} // classCommand
+    }
+}
