@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: viewCommand.java 1083 2003-12-03 08:30:23Z nfiedler $
+ * $Id: viewCommand.java 1533 2005-02-21 08:31:18Z nfiedler $
  *
  ********************************************************************/
 
@@ -31,6 +31,7 @@ import com.bluemarsh.jswat.ui.UIAdapter;
 import com.bluemarsh.jswat.util.Classes;
 import com.sun.jdi.ReferenceType;
 import com.sun.jdi.VirtualMachine;
+import java.io.File;
 import java.io.IOException;
 import java.util.List;
 
@@ -84,7 +85,10 @@ public class viewCommand extends JSwatCommand {
 
         if (source == null) {
             // Maybe the user entered a filename.
-            source = SourceFactory.getInstance().create(idClass, pathman);
+            File file = new File(idClass);
+            if (file.exists()) {
+                source = SourceFactory.getInstance().create(file, pathman);
+            }
         }
 
         if (source == null || !source.exists()) {
