@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: IdentifierNode.java 1458 2005-01-27 08:32:36Z nfiedler $
+ * $Id: IdentifierNode.java 1648 2005-04-13 07:16:20Z nfiedler $
  *
  ********************************************************************/
 
@@ -157,10 +157,10 @@ class IdentifierNode extends AbstractNode implements JoinableNode, VariableNode
             }
         } catch (AbsentInformationException aie) {
             throw new EvaluationException(
-                Bundle.getString("error.ident.failed"), aie);
+                Bundle.getString("error.ident.failed", aie), aie);
         } catch (ClassNotLoadedException cnle) {
             throw new EvaluationException(
-                Bundle.getString("error.ident.failed"), cnle);
+                Bundle.getString("error.ident.failed", cnle), cnle);
         }
     }
 
