@@ -1,6 +1,6 @@
 /*********************************************************************
  *
- *      Copyright (C) 2002-2003 Nathan Fiedler
+ *      Copyright (C) 2002-2004 Nathan Fiedler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: TreeBuilder.java 1082 2003-12-03 08:18:08Z nfiedler $
+ * $Id: TreeBuilder.java 1458 2005-01-27 08:32:36Z nfiedler $
  *
  ********************************************************************/
 
@@ -25,6 +25,7 @@ package com.bluemarsh.jswat.expr;
 import com.bluemarsh.jswat.parser.java.analysis.AnalysisAdapter;
 import com.bluemarsh.jswat.parser.java.node.*;
 import com.bluemarsh.jswat.util.Strings;
+import java.io.PrintStream;
 import java.util.EmptyStackException;
 import java.util.Stack;
 
@@ -35,15 +36,9 @@ import java.util.Stack;
  * @author  Nathan Fiedler
  */
 class TreeBuilder extends AnalysisAdapter {
-    /** Parser expects anything. */
-    private static final int START_STATE = 0;
-    /** Parser expects an argument. */
-    private static final int ARGUMENT_STATE = 1;
-    /** Parser expects an operator. */
-    private static final int OPERATOR_STATE = 2;
-    /** Parser expects an identifier to append to the previous one. */
-    private static final int APPEND_STATE = 3;
-    /** Current parser state, one of the STATE constants. */
+    /** The states this finite machine can exist in. */
+    private int ARGUMENT = 1, OPERATOR = 2, APPEND = 3;
+    /** Current parser state. */
     private int searchState;
     /** True if processing is completed, either because the end of
      * the expression was reached, or there was an error. */
@@ -59,6 +54,9 @@ class TreeBuilder extends AnalysisAdapter {
     /** Counter for assisting in validating the expression. If non-zero,
      * parsing the (n-1)th nested method invocation. */
     private int methodCount;
+    /** The last token we received (initially it's TWhitespace); used in
+     * determining if we are encountering a method invocation or not. */
+    private Token previousToken;
 
     // Infix expressions can be parsed from a series of tokens using two
     // stacks. One stack is used to hold parse trees under construction
@@ -129,10 +127,83 @@ class TreeBuilder extends AnalysisAdapter {
         }
     } // translateChar
 
+    /**
+     * Processes the given string, looking for character escapes and
+     * translating them to their actual values. Handles single character
+     * escapes, Unicode, and octal escapes.
+     *
+     * @param  str  string to be processed.
+     * @return  processed string.
+     */
+    protected static String translateString(String str) {
+        int strlen = str.length();
+        StringBuffer buf = new StringBuffer(strlen);
+        for (int ii = 0; ii < strlen; ii++) {
+            char ch = str.charAt(ii);
+            if (ch == '\\') {
+                ii++;
+                ch = str.charAt(ii);
+                if (ch == 'b') {
+                    buf.append('\b');
+                } else if (ch == 'f') {
+                    buf.append('\f');
+                } else if (ch == 't') {
+                    buf.append('\t');
+                } else if (ch == 'n') {
+                    buf.append('\n');
+                } else if (ch == 'r') {
+                    buf.append('\r');
+
+                } else if (ch == 'u') {
+                    // Unicode character.
+                    ii++;
+                    String hex = str.substring(ii, ii + 4);
+                    try {
+                        int i = Integer.parseInt(hex, 16);
+                        buf.append((char) i);
+                    } catch (NumberFormatException nfe) {
+                        throw new IllegalArgumentException(
+                            "invalid Unicode: " + hex);
+                    }
+                    ii += 3; // for loop will increment i again
+
+                } else if (ch >= '0' && ch <= '7') {
+                    // Octal escape.
+                    int jj = ii;
+                    while (jj < strlen) {
+                        char oc = str.charAt(jj);
+                        if (oc < '0' || oc > '7') {
+                            break;
+                        }
+                        jj++;
+                    }
+                    String octal = str.substring(ii, jj);
+                    try {
+                        int i = Integer.parseInt(octal, 8);
+                        buf.append((char) i);
+                    } catch (NumberFormatException nfe) {
+                        throw new IllegalArgumentException("invalid octal: "
+                                                           + octal);
+                    }
+                    ii = jj - 1; // for loop will increment i again
+
+                } else {
+                    buf.append(ch);
+                }
+            } else {
+                buf.append(ch);
+            }
+        }
+        return buf.toString();
+    } // translateString
+
     /**
      * Constructs a TreeBuilder.
+     *
+     * @param  context  parser context.
      */
     public TreeBuilder() {
+        searchState = ARGUMENT;
         rootNode = new RootNode();
         argumentStack = new Stack();
         operatorStack = new Stack();
@@ -147,6 +218,31 @@ class TreeBuilder extends AnalysisAdapter {
         return doneProcessing;
     } // doneProcessing
 
+    /**
+     * Dumps (and empties) the argument and operator stacks to the given
+     * output stream.
+     *
+     * @param  out  output stream to dump to.
+     */
+    void dumpStacks(PrintStream out) {
+        out.println("Argument stack...");
+        while (!argumentStack.empty()) {
+            Node n = (Node) argumentStack.pop();
+            out.print(n.getClass());
+            out.print(": ");
+            out.println(n.getToken().getText());
+        }
+
+        out.println();
+        out.println("Operator stack...");
+        while (!operatorStack.empty()) {
+            Node n = (Node) operatorStack.pop();
+            out.print(n.getClass());
+            out.print(": ");
+            out.println(n.getToken().getText());
+        }
+    } // dumpStacks
+
     /**
      * Returns the parser exception, if any.
      *
@@ -170,17 +266,29 @@ class TreeBuilder extends AnalysisAdapter {
      *
      * @param  lit  literal node.
      */
-    protected void handleLiteral(Node lit) {
-        if (searchState == APPEND_STATE) {
+    protected void handleLiteral(LiteralNode lit) {
+        if (searchState == APPEND) {
             setError(Errors.DOT_REQUIRES_ID, lit.getToken());
             return;
         }
-
-        // Push the literal onto the tree stack.
         argumentStack.push(lit);
-        searchState = OPERATOR_STATE;
+        searchState = OPERATOR;
     } // handleLiteral
 
+    /**
+     * Handles the given type node appropriately.
+     *
+     * @param  type  type node.
+     */
+    protected void handleType(TypeNode type) {
+        if (searchState == APPEND) {
+            setError(Errors.DOT_REQUIRES_ID, type.getToken());
+            return;
+        }
+        argumentStack.push(type);
+        searchState = OPERATOR;
+    } // handleType
+
     /**
      * Parse the string as a number, either integer or floating.
      *
@@ -258,7 +366,7 @@ class TreeBuilder extends AnalysisAdapter {
      * @param  op  operator node.
      */
     protected void handleOperator(OperatorNode op) {
-        if (searchState == APPEND_STATE) {
+        if (searchState == APPEND) {
             setError(Errors.DOT_REQUIRES_ID, op.getToken());
             return;
         }
@@ -269,14 +377,13 @@ class TreeBuilder extends AnalysisAdapter {
         // precedence: this will force left associativity). Otherwise
         // reduce the two stacks.
         if (!operatorStack.empty()) {
-            OperatorNode topop = (OperatorNode) operatorStack.peek();
-            int topopp = topop.precedence();
-            if (!topop.isSentinel() && op.precedence() >= topopp) {
+            OperatorNode top = (OperatorNode) operatorStack.peek();
+            if (!top.isSentinel() && op.precedence() >= top.precedence()) {
                 reduce();
             }
         }
         operatorStack.push(op);
-        searchState = ARGUMENT_STATE;
+        searchState = ARGUMENT;
     } // handleOperator
 
     /**
@@ -291,6 +398,17 @@ class TreeBuilder extends AnalysisAdapter {
         // the argument stack.
     } // handlePostfixOp
 
+    /**
+     * Handles the given variable node appropriately.
+     *
+     * @param  var variable node.
+     */
+    protected void handleVariable(Node var) {
+        // Push the variable onto the tree stack.
+        argumentStack.push(var);
+        searchState = OPERATOR;
+    } // handleVariable
+
     /**
      * Reduce the operator stack by one. If the operator stack top is
      * a left paren, no change is made.
@@ -303,29 +421,29 @@ class TreeBuilder extends AnalysisAdapter {
         // back on the argument stack. Note that the trees on the
         // argument stack represent the right and left arguments,
         // respectively.
-        OperatorNode topop = (OperatorNode) operatorStack.pop();
-        if (topop.isSentinel()) {
+        OperatorNode top = (OperatorNode) operatorStack.pop();
+        if (top.isSentinel()) {
             // Cleverly do nothing and let the caller handle it.
-        } else if (topop instanceof BinaryOperatorNode) {
+        } else if (top instanceof BinaryOperatorNode) {
             try {
                 Node arg2 = (Node) argumentStack.pop();
                 Node arg1 = (Node) argumentStack.pop();
-                topop.addChild(arg1);
-                topop.addChild(arg2);
-                argumentStack.push(topop);
+                top.addChild(arg1);
+                top.addChild(arg2);
+                argumentStack.push(top);
             } catch (EmptyStackException ese) {
-                setError(Errors.MISSING_ARGS, topop.getToken());
+                setError(Errors.MISSING_ARGS, top.getToken());
             }
-        } else if (topop instanceof UnaryOperatorNode) {
+        } else if (top instanceof UnaryOperatorNode) {
             try {
                 Node arg = (Node) argumentStack.pop();
-                topop.addChild(arg);
-                argumentStack.push(topop);
+                top.addChild(arg);
+                argumentStack.push(top);
             } catch (EmptyStackException ese) {
-                setError(Errors.MISSING_ARGS, topop.getToken());
+                setError(Errors.MISSING_ARGS, top.getToken());
             }
         } else {
-            setError(Errors.UNEXPECTED_TOKEN, topop.getToken());
+            setError(Errors.UNEXPECTED_TOKEN, top.getToken());
         }
     } // reduce
 
@@ -357,7 +475,7 @@ class TreeBuilder extends AnalysisAdapter {
      * @param  add    additional error suffix.
      */
     protected void setError(int error, String add) {
-        setError(Errors.getMessage(error) + ' ' + add);
+        setError(Errors.getMessage(error) + ": " + add);
     } // setError
 
     /**
@@ -368,79 +486,43 @@ class TreeBuilder extends AnalysisAdapter {
      * @param  token  token providing additional details.
      */
     protected void setError(int error, Token token) {
-        setError(Errors.getMessage(error) + ' ' + token.getText()
+        setError(Errors.getMessage(error) + ": " + token.getText()
                  + " @ " + token.getPos());
     } // setError
 
     /**
-     * Processes the given string, looking for character escapes and
-     * translating them to their actual values. Handles single character
-     * escapes, Unicode, and octal escapes.
+     * Set the error value and set the done flag to true.
+     * Appends the token information to the message.
      *
-     * @param  str  string to be processed.
-     * @return  processed string.
+     * @param  msg    error message.
+     * @param  token  token providing additional details.
      */
-    protected String translateString(String str) {
-        int strlen = str.length();
-        StringBuffer buf = new StringBuffer(strlen);
-        for (int ii = 0; ii < strlen; ii++) {
-            char ch = str.charAt(ii);
-            if (ch == '\\') {
-                ii++;
-                ch = str.charAt(ii);
-                if (ch == 'b') {
-                    buf.append('\b');
-                } else if (ch == 'f') {
-                    buf.append('\f');
-                } else if (ch == 't') {
-                    buf.append('\t');
-                } else if (ch == 'n') {
-                    buf.append('\n');
-                } else if (ch == 'r') {
-                    buf.append('\r');
+    protected void setError(String msg, Token token) {
+        setError(msg + ": " + token.getText() + " @ " + token.getPos());
+    } // setError
 
-                } else if (ch == 'u') {
-                    // Unicode character.
-                    ii++;
-                    String hex = str.substring(ii, ii + 4);
-                    try {
-                        int i = Integer.parseInt(hex, 16);
-                        buf.append((char) i);
-                    } catch (NumberFormatException nfe) {
-                        throw new IllegalArgumentException(
-                            "invalid Unicode: " + hex);
-                    }
-                    ii += 3; // for loop will increment i again
+    /**
+     * Set the error value and set the done flag to true.
+     * Appends the token information to the message.
+     *
+     * @param  error  error value.
+     * @param  token  token providing additional details.
+     * @param  add    additional error message.
+     */
+    protected void setError(int error, Token token, String add) {
+        setError(Errors.getMessage(error) + ": " + token.getText()
+                 + " @ " + token.getPos() + ": " + add);
+    } // setError
 
-                } else if (ch >= '0' && ch <= '7') {
-                    // Octal escape.
-                    int jj = ii;
-                    while (jj < strlen) {
-                        char oc = str.charAt(jj);
-                        if (oc < '0' || oc > '7') {
-                            break;
-                        }
-                        jj++;
-                    }
-                    String octal = str.substring(ii, jj);
-                    try {
-                        int i = Integer.parseInt(octal, 8);
-                        buf.append((char) i);
-                    } catch (NumberFormatException nfe) {
-                        throw new IllegalArgumentException("invalid octal: "
-                                                           + octal);
-                    }
-                    ii = jj - 1; // for loop will increment i again
-
-                } else {
-                    buf.append(ch);
-                }
-            } else {
-                buf.append(ch);
-            }
-        }
-        return buf.toString();
-    } // translateString
+    /**
+     * Sets the token that was just applied, which means it is the
+     * previous token for the next application.
+     *
+     * @param  token  the previous token.
+     */
+    protected void setPrevToken(Token token) {
+        previousToken = token;
+    } // setPrevToken
 
     //
     // Methods inherited from AnalysisAdapter.
@@ -451,7 +533,7 @@ class TreeBuilder extends AnalysisAdapter {
      */
     public void caseEOF(EOF node) {
         doneProcessing = true;
-        if (searchState == APPEND_STATE) {
+        if (searchState == APPEND) {
             setError(Errors.DOT_REQUIRES_ID, node);
             return;
         }
@@ -463,22 +545,22 @@ class TreeBuilder extends AnalysisAdapter {
 
         int count = 0;
         while (!operatorStack.empty()) {
-            OperatorNode topop = (OperatorNode) operatorStack.peek();
-            if (topop instanceof LeftParen) {
-                setError(Errors.UNMATCHED_LPAREN, topop.getToken());
+            OperatorNode top = (OperatorNode) operatorStack.peek();
+            if (top instanceof LeftParen) {
+                setError(Errors.UNMATCHED_LPAREN, top.getToken());
                 return;
-            } else if (topop instanceof LeftBracket) {
-                setError(Errors.UNMATCHED_LBRACKET, topop.getToken());
+            } else if (top instanceof LeftBracket) {
+                setError(Errors.UNMATCHED_LBRACKET, top.getToken());
                 return;
-            } else if (topop.isSentinel()) {
-                setError(Errors.INVALID_EXPR, topop.getToken());
+            } else if (top.isSentinel()) {
+                setError(Errors.INVALID_EXPR, top.getToken());
                 return;
             }
             reduce();
             count++;
-            // Reduce no more than 100 times.
-            if (count > 100) {
-                break;
+            if (count > 500) {
+                setError(Errors.LARGE_OPER_STACK);
+                return;
             }
         }
 
@@ -487,7 +569,7 @@ class TreeBuilder extends AnalysisAdapter {
             if (argumentStack.empty() && operatorStack.empty()) {
                 rootNode.addChild(topArg);
             } else {
-                setError(Errors.INVALID_EXPR, " Argument stack not empty.");
+                setError(Errors.ARG_STACK_NON_EMPTY, topArg.getToken());
             }
         }
     }
@@ -498,17 +580,31 @@ class TreeBuilder extends AnalysisAdapter {
 
     /**
      *
-     * @param  node  identifier node.
+     * @param  token  identifier token.
      */
-    public void caseTIdentifier(TIdentifier node) {
-        String name = node.getText();
-        if (searchState == APPEND_STATE) {
-            // Assumes stack is non-empty.
-            IdentifierNode idn = (IdentifierNode) argumentStack.peek();
-            idn.append(node.getText());
-            searchState = OPERATOR_STATE;
+    public void caseTIdentifier(TIdentifier token) {
+        String name = token.getText();
+        if (searchState == APPEND) {
+            // Both argument and operator stacks are non-empty at this point.
+            Node node = (Node) argumentStack.peek();
+            if (node instanceof JoinableNode) {
+                Node opr = (Node) operatorStack.peek();
+                if (opr instanceof JoinOperatorNode) {
+                    // Add the second operand for our join operator.
+                    handleVariable(new IdentifierNode(token, name));
+                    // Reduce the operator stack once to get the join
+                    // operator onto the argument stack where it belongs.
+                    reduce();
+                    searchState = OPERATOR;
+                } else {
+                    setError(Errors.UNKNOWN_STATE, token, "append without join");
+                }
+            } else {
+                setError(Errors.UNKNOWN_STATE, token, "cannot append to "
+                    + node.getToken().getText());
+            }
         } else {
-            handleLiteral(new IdentifierNode(node, name));
+            handleVariable(new IdentifierNode(token, name));
         }
     }
 
@@ -517,11 +613,12 @@ class TreeBuilder extends AnalysisAdapter {
      * @param  node  'this' node.
      */
     public void caseTThis(TThis node) {
-        if (searchState == APPEND_STATE) {
+        if (searchState == APPEND) {
+            // this is where RFE 881 implementation would begin
             setError(Errors.DOT_REQUIRES_ID, node);
             return;
         }
-        handleLiteral(new IdentifierNode(node, "this"));
+        handleVariable(new IdentifierNode(node, "this"));
     }
 
     //
@@ -634,15 +731,27 @@ class TreeBuilder extends AnalysisAdapter {
      * @param  node  left parenthesis.
      */
     public void caseTLParenthese(TLParenthese node) {
+        // Append state is definitely wrong, but argument and operator
+        // states are perfectly exceptable.
+        if (searchState == APPEND) {
+            setError(Errors.DOT_REQUIRES_ID, node);
+            return;
+        }
         LeftParen lp = new LeftParen(node);
-        // Simply push it on the operator stack; the search should
-        // have been for an argument and should remain so.
-        if (!argumentStack.empty()
-            && argumentStack.peek() instanceof IdentifierNode) {
-            // This is the start of a method invocation.
-            IdentifierNode ident = (IdentifierNode) argumentStack.pop();
-            MethodNode method = new MethodNode(ident.getToken());
-            method.addChild(ident);
+        // If the last token was an identifier, then this is a method call.
+        if (previousToken instanceof TIdentifier) {
+            // The argument stack is assumed to be empty.
+            Node n = (Node) argumentStack.pop();
+            MethodNode method = null;
+            if (n instanceof IdentifierNode) {
+                IdentifierNode inode = (IdentifierNode) n;
+                method = new MethodNode(inode.getToken(), inode.getIdentifier());
+            } else {
+                JoinOperatorNode onode = (JoinOperatorNode) n;
+                Node object = onode.getChild(0);
+                String name = onode.getChild(1).getToken().getText();
+                method = new MethodNode(onode.getToken(), object, name);
+            }
             // Put it on the argument stack as a sentinel, to mark
             // the beginning of the method arguments.
             argumentStack.push(method);
@@ -650,10 +759,10 @@ class TreeBuilder extends AnalysisAdapter {
             // right parenthesis, we can determine that we were
             // making a method call.
             operatorStack.push(method);
-            searchState = ARGUMENT_STATE;
+            searchState = ARGUMENT;
             methodCount++;
         }
-        // Else, must be the start of a type-cast.
+        // Else, it is the start of a type-cast or a subgroup.
         operatorStack.push(lp);
     }
 
@@ -668,15 +777,16 @@ class TreeBuilder extends AnalysisAdapter {
         }
         // If there is a left parenthesis on the operator stack, we can
         // "cancel" the pair. If the operator stack contains some other
-        // operator on top, reduce the stacks.
-        OperatorNode topop = (OperatorNode) operatorStack.peek();
-        while (!(topop instanceof LeftParen)) {
+        // operator on top, reduce the stacks. This also covers the case
+        // where the parentheses were used for grouping only.
+        OperatorNode top = (OperatorNode) operatorStack.peek();
+        while (!(top instanceof LeftParen)) {
             reduce();
-            if (operatorStack.empty() || topop.isSentinel()) {
+            if (operatorStack.empty() || top.isSentinel()) {
                 setError(Errors.UNMATCHED_RPAREN, node);
                 return;
             }
-            topop = (OperatorNode) operatorStack.peek();
+            top = (OperatorNode) operatorStack.peek();
         }
         operatorStack.pop();
 
@@ -705,19 +815,30 @@ class TreeBuilder extends AnalysisAdapter {
             // Maybe it is a type-cast operation; otherwise it was a
             // grouping operator and that has been taken care of.
             try {
-                // Pop off the type for the type-cast.
                 Node n = (Node) argumentStack.peek();
-                if (n instanceof IdentifierNode) {
+                if (n instanceof TypeNode) {
+                    argumentStack.pop();
+                    TypeNode tn = (TypeNode) n;
+                    TypeCastOperatorNode tcon = new TypeCastOperatorNode(
+                        tn.getToken(), tn.getTypeName());
+                    handleOperator(tcon);
+                } else if (n instanceof IdentifierNode) {
                     argumentStack.pop();
                     IdentifierNode in = (IdentifierNode) n;
                     TypeCastOperatorNode tcon = new TypeCastOperatorNode(
-                        in.getToken());
-                    tcon.addChild(in);
-                    // The type-cast is an operator.
+                        in.getToken(), in.getIdentifier());
+                    handleOperator(tcon);
+                } else if (n instanceof JoinOperatorNode) {
+                    argumentStack.pop();
+                    JoinOperatorNode jon = (JoinOperatorNode) n;
+                    TypeCastOperatorNode tcon = new TypeCastOperatorNode(
+                        jon.getToken(), jon.mergeChildren());
                     handleOperator(tcon);
                 }
+            } catch (EvaluationException ee) {
+                setError(ee.getMessage(), top.getToken());
             } catch (EmptyStackException ese) {
-                setError(Errors.MISSING_ARGS, topop.getToken());
+                setError(Errors.MISSING_ARGS, top.getToken());
             }
         }
     }
@@ -727,21 +848,28 @@ class TreeBuilder extends AnalysisAdapter {
      * @param  node  left bracket.
      */
     public void caseTLBracket(TLBracket node) {
-        LeftBracket lb = new LeftBracket(node);
-        // Simply push it on the operator stack; the search should
-        // have been for an argument and should remain so.
-        if (!argumentStack.empty()
-            && argumentStack.peek() instanceof IdentifierNode) {
-            // This is the start of an array access.
-            IdentifierNode ident = (IdentifierNode) argumentStack.pop();
-            ArrayRefNode arrayref = new ArrayRefNode(ident.getToken());
-            arrayref.addChild(ident);
-            // Put it on both stacks.
-            argumentStack.push(arrayref);
-            operatorStack.push(arrayref);
-            searchState = ARGUMENT_STATE;
+        // Make sure there is something reasonable on the stack, since
+        // a left bracket without a preceding type or identifier is
+        // incorrect syntax.
+        if (argumentStack.isEmpty()) {
+            setError(Errors.UNEXPECTED_TOKEN, node);
+        } else {
+            Node n = (Node) argumentStack.peek();
+            if (!(n instanceof JoinOperatorNode)
+                && !(n instanceof IdentifierNode)
+                && !(n instanceof TypeNode)) {
+                setError(Errors.UNEXPECTED_TOKEN, node);
+            }
         }
+        // Push the left bracket on the operator stack. We can't tell yet
+        // if this is going to be a typecast or an array access.
+        Node lb = new LeftBracket(node);
         operatorStack.push(lb);
+        // Put it on the argument stack as a sentinel for the array index,
+        // if that is indeed what this turns out to be.
+        argumentStack.push(lb);
+        // We may have been looking for an argument or an operator,
+        // and we can't be sure what we will find next.
     }
 
     /**
@@ -750,44 +878,49 @@ class TreeBuilder extends AnalysisAdapter {
      */
     public void caseTRBracket(TRBracket node) {
         // If there is a left bracket on the operator stack, we can
-        // "cancel" the pair If the operator stack contains some other
+        // "cancel" the pair. If the operator stack contains some other
         // operator on top, reduce the stacks.
         if (operatorStack.empty()) {
             setError(Errors.UNMATCHED_RBRACKET, node);
             return;
         }
-        OperatorNode topop = (OperatorNode) operatorStack.peek();
-        while (!(topop instanceof LeftBracket)) {
+        OperatorNode top = (OperatorNode) operatorStack.peek();
+        while (!(top instanceof LeftBracket)) {
             reduce();
-            if (operatorStack.empty() || topop.isSentinel()) {
+            if (operatorStack.empty() || top.isSentinel()) {
                 setError(Errors.UNMATCHED_RBRACKET, node);
                 return;
             }
-            topop = (OperatorNode) operatorStack.peek();
+            top = (OperatorNode) operatorStack.peek();
         }
         operatorStack.pop();
 
-        // Now check for the array reference case.
-        if (!operatorStack.empty()
-            && operatorStack.peek() instanceof ArrayRefNode) {
-            ArrayRefNode arrayref = (ArrayRefNode) operatorStack.pop();
-            // It was an array reference.
-            // Pop off the arguments until the [ is found.
-            Node n = (Node) argumentStack.pop();
-            int count = 0;
-            while (n != arrayref) {
-                arrayref.addChild(n);
-                count++;
-                n = (Node) argumentStack.pop();
-            }
-            if (count > 1) {
-                setError(Errors.ARRAY_MULTI_INDEX, arrayref.getToken());
-            }
-            // Put the array reference back on the argument stack.
-            argumentStack.push(arrayref);
+        // Was there anything between the brackets?
+        // (we know there will be a left bracket and something else on the
+        // argument stack, given that we are here).
+        Node index = (Node) argumentStack.pop();
+        Node name = (Node) argumentStack.pop();
+        if (index instanceof LeftBracket) {
+            // It was probably part of a typecast, but we can't be sure.
+            // In any case, make a TypeNode out of this and put it on
+            // the argument stack.
+            Token token = name.getToken();
+            argumentStack.push(new TypeNode(token, token.getText() + "[]"));
         } else {
-            // Brackets without a preceeding identifier.
-            setError(Errors.UNEXPECTED_TOKEN, topop.getToken());
+            // It was an array reference.
+            // Make sure that only one array index was provided.
+            if (name instanceof LeftBracket) {
+                ArrayNode arrayref = new ArrayNode(name.getToken());
+                // Retrieve the thing that was there before the left bracket
+                // (an identifier or type).
+                name = (Node) argumentStack.pop();
+                arrayref.addChild(name);
+                arrayref.addChild(index);
+                // Put the array reference on the argument stack; it's a value.
+                argumentStack.push(arrayref);
+            } else {
+                setError(Errors.ARRAY_MULTI_INDEX, name.getToken());
+            }
         }
     }
 
@@ -804,10 +937,10 @@ class TreeBuilder extends AnalysisAdapter {
                 setError(Errors.UNEXPECTED_TOKEN, node);
                 return;
             }
-            OperatorNode topop = (OperatorNode) operatorStack.peek();
-            while (!topop.isSentinel()) {
+            OperatorNode top = (OperatorNode) operatorStack.peek();
+            while (!top.isSentinel()) {
                 reduce();
-                topop = (OperatorNode) operatorStack.peek();
+                top = (OperatorNode) operatorStack.peek();
             }
         }
         // Was there anything before this comma?
@@ -821,22 +954,11 @@ class TreeBuilder extends AnalysisAdapter {
      */
     public void caseTDot(TDot node) {
         // Dot is special and needs processing before evaluation.
-        if (!argumentStack.empty()) {
-            Node n = (Node) argumentStack.peek();
-            if (n instanceof IdentifierNode) {
-                // If top argument is an identifier, append the dot
-                // to the identifier's name.
-                IdentifierNode idn = (IdentifierNode) n;
-                idn.append(node.getText());
-                // The next identifier we see must be appended as well.
-                searchState = APPEND_STATE;
-            } else if (n instanceof LiteralNode) {
-                // If top argument is a string literal, this starts a
-                // method invocation on that string.
-                setError(Errors.UNSUPPORTED_FEATURE, node);
-            } else {
-                setError(Errors.UNEXPECTED_TOKEN, node);
-            }
+        if (!argumentStack.empty()
+            && argumentStack.peek() instanceof JoinableNode) {
+            // We have a joinable, which makes this a join in progress.
+            handleOperator(new JoinOperatorNode(node));
+            searchState = APPEND;
         } else {
             setError(Errors.UNEXPECTED_TOKEN, node);
         }
@@ -849,7 +971,7 @@ class TreeBuilder extends AnalysisAdapter {
     public void caseTPlus(TPlus node) {
         // The plus is unary if we are expecting an argument or if
         // the argument stack is empty (ie. search state is 'start').
-        if (searchState == START_STATE || searchState == ARGUMENT_STATE) {
+        if (searchState == ARGUMENT) {
             handleOperator(new PlusUnaryOperatorNode(node));
         } else {
             handleOperator(new PlusBinaryOperatorNode(node));
@@ -863,7 +985,7 @@ class TreeBuilder extends AnalysisAdapter {
     public void caseTMinus(TMinus node) {
         // The minus is unary if we are expecting an argument or if
         // the argument stack is empty (ie. search state is 'start').
-        if (searchState == START_STATE || searchState == ARGUMENT_STATE) {
+        if (searchState == ARGUMENT) {
             handleOperator(new MinusUnaryOperatorNode(node));
         } else {
             handleOperator(new MinusBinaryOperatorNode(node));
@@ -1022,6 +1144,82 @@ class TreeBuilder extends AnalysisAdapter {
         handleOperator(new GtEqOperatorNode(node));
     }
 
+    //
+    // Primitive data types
+    //
+
+    /**
+     *
+     * @param  node  boolean keyword.
+     */
+    public void caseTBoolean(TBoolean node) {
+        handleType(new TypeNode(node, "boolean"));
+    }
+
+    /**
+     *
+     * @param  node  byte keyword.
+     */
+    public void caseTByte(TByte node) {
+        handleType(new TypeNode(node, "byte"));
+    }
+
+    /**
+     *
+     * @param  node  char keyword
+     */
+    public void caseTChar(TChar node) {
+        handleType(new TypeNode(node, "char"));
+    }
+
+    /**
+     *
+     * @param  node  class keyword
+     */
+    public void caseTClass(TClass node) {
+        setError(Errors.UNSUPPORTED_FEATURE, node);
+    }
+
+    /**
+     *
+     * @param  node  double keyword
+     */
+    public void caseTDouble(TDouble node) {
+        handleType(new TypeNode(node, "double"));
+    }
+
+    /**
+     *
+     * @param  node  float keyword
+     */
+    public void caseTFloat(TFloat node) {
+        handleType(new TypeNode(node, "float"));
+    }
+
+    /**
+     *
+     * @param  node  int keyword
+     */
+    public void caseTInt(TInt node) {
+        handleType(new TypeNode(node, "int"));
+    }
+
+    /**
+     *
+     * @param  node  long keyword
+     */
+    public void caseTLong(TLong node) {
+        handleType(new TypeNode(node, "long"));
+    }
+
+    /**
+     *
+     * @param  node  short keyword
+     */
+    public void caseTShort(TShort node) {
+        handleType(new TypeNode(node, "short"));
+    }
+
     //
     // Unsupported features (that may be supported in the future)
     //
@@ -1050,86 +1248,6 @@ class TreeBuilder extends AnalysisAdapter {
         setError(Errors.UNSUPPORTED_FEATURE, node);
     }
 
-    /**
-     *
-     * @param  node  boolean keyword.
-     */
-    public void caseTBoolean(TBoolean node) {
-        // Treat this as an identifier.
-        handleLiteral(new IdentifierNode(node, "boolean"));
-    }
-
-    /**
-     *
-     * @param  node  byte keyword.
-     */
-    public void caseTByte(TByte node) {
-        // Treat this as an identifier.
-        handleLiteral(new IdentifierNode(node, "byte"));
-    }
-
-    /**
-     *
-     * @param  node  char keyword
-     */
-    public void caseTChar(TChar node) {
-        // Treat this as an identifier.
-        handleLiteral(new IdentifierNode(node, "char"));
-    }
-
-    /**
-     *
-     * @param  node  class keyword
-     */
-    public void caseTClass(TClass node) {
-        setError(Errors.UNSUPPORTED_FEATURE, node);
-    }
-
-    /**
-     *
-     * @param  node  double keyword
-     */
-    public void caseTDouble(TDouble node) {
-        // Treat this as an identifier.
-        handleLiteral(new IdentifierNode(node, "double"));
-    }
-
-    /**
-     *
-     * @param  node  float keyword
-     */
-    public void caseTFloat(TFloat node) {
-        // Treat this as an identifier.
-        handleLiteral(new IdentifierNode(node, "float"));
-    }
-
-    /**
-     *
-     * @param  node  int keyword
-     */
-    public void caseTInt(TInt node) {
-        // Treat this as an identifier.
-        handleLiteral(new IdentifierNode(node, "int"));
-    }
-
-    /**
-     *
-     * @param  node  long keyword
-     */
-    public void caseTLong(TLong node) {
-        // Treat this as an identifier.
-        handleLiteral(new IdentifierNode(node, "long"));
-    }
-
-    /**
-     *
-     * @param  node  short keyword
-     */
-    public void caseTShort(TShort node) {
-        // Treat this as an identifier.
-        handleLiteral(new IdentifierNode(node, "short"));
-    }
-
     //
     // Unsupported tokens.
     //
@@ -1491,7 +1609,7 @@ class TreeBuilder extends AnalysisAdapter {
      * @param  node  assign keyword.
      */
     public void caseTAssign(TAssign node) {
-        setError(Errors.UNSUPPORTED_TOKEN, node);
+        handleOperator(new AssignOperatorNode(node));
     }
 
     /**
