@@ -1,6 +1,6 @@
 /*********************************************************************
  *
- *      Copyright (C) 1999-2004 Nathan Fiedler
+ *      Copyright (C) 1999-2005 Nathan Fiedler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: BreakPanel.java 1313 2004-09-29 05:20:24Z nfiedler $
+ * $Id: BreakPanel.java 1768 2005-07-03 23:18:52Z nfiedler $
  *
  ********************************************************************/
 
@@ -104,7 +104,7 @@ public class BreakPanel extends AbstractPanel
         uiTree.setCellRenderer(new BreakpointRenderer());
         uiTree.addMouseListener(this);
         uiComponent = buildUI(new JScrollPane(uiTree));
-    } // BreakPanel
+    }
 
     /**
      * Invoked when a breakpoint has been added.
@@ -113,7 +113,7 @@ public class BreakPanel extends AbstractPanel
      */
     public void breakpointAdded(BreakpointEvent e) {
         refreshLater();
-    } // breakpointAdded
+    }
 
     /**
      * Invoked when a breakpoint has been modified.
@@ -122,7 +122,7 @@ public class BreakPanel extends AbstractPanel
      */
     public void breakpointModified(BreakpointEvent e) {
         refreshLater();
-    } // breakpointModified
+    }
 
     /**
      * Invoked when a breakpoint has been removed.
@@ -133,7 +133,7 @@ public class BreakPanel extends AbstractPanel
         // We're notified before the breakpoint is actually removed,
         // so call the refresh() method later.
         refreshLater();
-    } // breakpointRemoved
+    }
 
     /**
      * Build the interface components for this panel (buttons mostly).
@@ -238,7 +238,7 @@ public class BreakPanel extends AbstractPanel
         newComp.add(button);
 
         return newComp;
-    } // buildUI
+    }
 
     /**
      * Fill in the given tree node with the breakpoint's contents.
@@ -266,7 +266,7 @@ public class BreakPanel extends AbstractPanel
             BreakpointNode brknode = new BreakpointNode(bp);
             node.insert(brknode, node.getChildCount());
         }
-    } // buildGroupNode
+    }
 
     /**
      * Called when the Session is about to be closed.
@@ -280,7 +280,7 @@ public class BreakPanel extends AbstractPanel
         brkman.removeBreakListener(this);
         brkman.removeGroupListener(this);
         super.closing(sevt);
-    } // closing
+    }
 
     /**
      * Deletes the currently selected breakpoints and groups.
@@ -305,7 +305,7 @@ public class BreakPanel extends AbstractPanel
                     return false;
                 }
             });
-    } // deleteSelections
+    }
 
     /**
      * Disables the currently selected breakpoints and groups.
@@ -326,7 +326,7 @@ public class BreakPanel extends AbstractPanel
                     return false;
                 }
             });
-    } // disableSelections
+    }
 
     /**
      * Show the given breakpoint's properties.
@@ -346,7 +346,7 @@ public class BreakPanel extends AbstractPanel
                 }
             });
         editor.setVisible(true);
-    } // editBreakpoint
+    }
 
     /**
      * Enables the currently selected breakpoints and groups.
@@ -367,7 +367,7 @@ public class BreakPanel extends AbstractPanel
                     return false;
                 }
             });
-    } // enableSelections
+    }
 
     /**
      * Returns the first currently selected breakpoint in the panel,
@@ -386,7 +386,7 @@ public class BreakPanel extends AbstractPanel
             }
         }
         return null;
-    } // getSelectedBreakpoint
+    }
 
     /**
      * Returns a reference to the UI component. This is the object
@@ -397,7 +397,7 @@ public class BreakPanel extends AbstractPanel
      */
     public JComponent getUI() {
         return uiComponent;
-    } // getUI
+    }
 
     /**
      * Invoked when a group has been added.
@@ -406,7 +406,7 @@ public class BreakPanel extends AbstractPanel
      */
     public void groupAdded(GroupEvent event) {
         refreshLater();
-    } // groupAdded
+    }
 
     /**
      * Invoked when a group has been disabled.
@@ -415,7 +415,7 @@ public class BreakPanel extends AbstractPanel
      */
     public void groupDisabled(GroupEvent event) {
         refreshLater();
-    } // groupDisabled
+    }
 
     /**
      * Invoked when a group has been enabled.
@@ -424,7 +424,7 @@ public class BreakPanel extends AbstractPanel
      */
     public void groupEnabled(GroupEvent event) {
         refreshLater();
-    } // groupEnabled
+    }
 
     /**
      * Invoked when a group has been removed.
@@ -433,7 +433,7 @@ public class BreakPanel extends AbstractPanel
      */
     public void groupRemoved(GroupEvent event) {
         refreshLater();
-    } // groupRemoved
+    }
 
     /**
      * Invoked when the mouse button has been clicked (pressed and
@@ -459,7 +459,7 @@ public class BreakPanel extends AbstractPanel
                 }
             }
         }
-    } // mouseClicked
+    }
 
     /**
      * Invoked when a mouse button has been pressed on a component.
@@ -502,7 +502,7 @@ public class BreakPanel extends AbstractPanel
         brkman.addBreakListener(this);
         brkman.addGroupListener(this);
         UIManager.addPropertyChangeListener(this);
-    } // opened
+    }
 
     /**
      * Perform some action on the set of selected breakpoints
@@ -541,7 +541,7 @@ public class BreakPanel extends AbstractPanel
         if (rebuild) {
             refreshLater();
         }
-    } // processSelections
+    }
 
     /**
      * This method gets called when a bound property is changed.
@@ -556,7 +556,7 @@ public class BreakPanel extends AbstractPanel
                 ((JComponent) renderer).updateUI();
             }
         }
-    } // propertyChange
+    }
 
     /**
      * Update the display on the screen. Use the given Session to fetch
@@ -621,7 +621,7 @@ public class BreakPanel extends AbstractPanel
             };
         }
         uiTree.addTreeExpansionListener(treeExpansionListener);
-    } // refresh
+    }
 
     /**
      * Do something about the tree expansion.
@@ -638,7 +638,7 @@ public class BreakPanel extends AbstractPanel
             BreakpointGroup grp = (BreakpointGroup) o;
             grp.setExpanded(expanded);
         }
-    } // handlePathExpanded
+    }
 
     /**
      * Show the source code for the first selected breakpoint,
@@ -651,16 +651,18 @@ public class BreakPanel extends AbstractPanel
             return;
         }
 
-        // First try the way that is most likely to succeed.
+        // Use most accurate method first, then last accurate methods.
         LocatableBreakpoint lbp = (LocatableBreakpoint) bp;
         PathManager pathman = (PathManager)
             owningSession.getManager(PathManager.class);
         SourceSource src = null;
-        // This won't work if the class name is wild-carded.
-        try {
-            src = pathman.mapSource(lbp.getClassName());
-        } catch (IOException ioe) {
-            // This case is handled below.
+        ReferenceType clazz = lbp.getReferenceType();
+        if (clazz != null) {
+            try {
+                src = pathman.mapSource(clazz);
+            } catch (IOException ioe) {
+                // ignore and fall through
+            }
         }
 
         if (src == null) {
@@ -680,18 +682,11 @@ public class BreakPanel extends AbstractPanel
             }
         }
         if (src == null) {
-            // That didn't work, try the resolved location.
-            ReferenceType clazz = lbp.getReferenceType();
-            if (clazz == null) {
-                owningSession.getUIAdapter().showMessage(
-                    UIAdapter.MESSAGE_ERROR,
-                    Bundle.getString("Break.bpHasNoLocation"));
-                return;
-            }
+            // This won't work if the class name is wild-carded.
             try {
-                src = pathman.mapSource(clazz);
+                src = pathman.mapSource(lbp.getClassName());
             } catch (IOException ioe) {
-                // This case is handled below.
+                // ignore and fall through
             }
         }
 
@@ -713,7 +708,7 @@ public class BreakPanel extends AbstractPanel
             owningSession.getUIAdapter().showMessage(
                 UIAdapter.MESSAGE_ERROR, msg);
         }
-    } // showSource
+    }
 
     /**
      * Class BreakpointRenderer implements a tree cell renderer that
@@ -730,7 +725,7 @@ public class BreakPanel extends AbstractPanel
             setClosedIcon(null);
             setLeafIcon(null);
             setOpenIcon(null);
-        } // BreakpointRenderer
+        }
 
         /**
          * Return the component for rendering the identified cell.
@@ -757,7 +752,7 @@ public class BreakPanel extends AbstractPanel
             }
             setFont(node.getFont(font));
             return this;
-        } // getTreeCellRendererComponent
+        }
 
         /**
          * Called when the look and feel is changing.
@@ -775,8 +770,8 @@ public class BreakPanel extends AbstractPanel
                                                "Tree.textBackground"));
             setBorderSelectionColor(UIManager.getColor(
                                         "Tree.selectionBorderColor"));
-        } // updateUI
-    } // BreakpointRenderer
+        }
+    }
 
     /**
      * Class BasicTreeNode is the base class for both breakpoint
@@ -791,7 +786,7 @@ public class BreakPanel extends AbstractPanel
          */
         public BasicTreeNode(Object userObject) {
             super(userObject);
-        } // BasicTreeNode
+        }
 
         /**
          * Constructs a BasicTreeNode.
@@ -801,7 +796,7 @@ public class BreakPanel extends AbstractPanel
          */
         public BasicTreeNode(Object userObject, boolean allowsChildren) {
             super(userObject, allowsChildren);
-        } // BasicTreeNode
+        }
 
         /**
          * Returns the font appropriate for this node, based on the
@@ -813,8 +808,8 @@ public class BreakPanel extends AbstractPanel
          */
         public Font getFont(Font font) {
             return null;
-        } // getFont
-    } // BasicTreeNode
+        }
+    }
 
     /**
      * Class BreakpointNode represents a breakpoint in the tree.
@@ -828,7 +823,7 @@ public class BreakPanel extends AbstractPanel
          */
         public BreakpointNode(Breakpoint brk) {
             super(brk, false);
-        } // BreakpointNode
+        }
 
         /**
          * Returns the font appropriate for this node, based on the
@@ -847,7 +842,7 @@ public class BreakPanel extends AbstractPanel
                 font = font.deriveFont(Font.BOLD | font.getStyle());
             }
             return font;
-        } // getFont
+        }
 
         /**
          * Returns the name of the breakpoint group.
@@ -858,8 +853,8 @@ public class BreakPanel extends AbstractPanel
             Breakpoint bp = (Breakpoint) getUserObject();
             // Get the terse breakpoint description.
             return bp.toString(true);
-        } // toString
-    } // BreakpointNode
+        }
+    }
 
     /**
      * Class GroupNode represents a breakpoint group in the tree.
@@ -873,7 +868,7 @@ public class BreakPanel extends AbstractPanel
          */
         public GroupNode(BreakpointGroup brkgrp) {
             super(brkgrp);
-        } // GroupNode
+        }
 
         /**
          * Returns the font appropriate for this node, based on the
@@ -889,7 +884,7 @@ public class BreakPanel extends AbstractPanel
                 font = font.deriveFont(Font.ITALIC);
             }
             return font;
-        } // getFont
+        }
 
         /**
          * Returns the name of the breakpoint group.
@@ -899,8 +894,8 @@ public class BreakPanel extends AbstractPanel
         public String toString() {
             BreakpointGroup brkgrp = (BreakpointGroup) getUserObject();
             return brkgrp.getName();
-        } // toString
-    } // GroupNode
+        }
+    }
 
     /**
      * Interface NodeProcessor defines methods necessary for a class
@@ -916,5 +911,5 @@ public class BreakPanel extends AbstractPanel
          * @return  true if the tree should be rebuilt.
          */
         boolean processNode(BreakpointManager brkman, Object o);
-    } // NodeProcessor
-} // BreakPanel
+    }
+}
