@@ -1,6 +1,6 @@
 /*********************************************************************
  *
- *      Copyright (C) 1999-2004 Nathan Fiedler
+ *      Copyright (C) 1999-2005 Nathan Fiedler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: ClassPanel.java 1292 2004-09-15 05:10:22Z nfiedler $
+ * $Id: ClassPanel.java 1767 2005-07-03 01:19:50Z nfiedler $
  *
  ********************************************************************/
 
@@ -130,7 +130,7 @@ public class ClassPanel extends AbstractPanel
             dtcr.setLeafIcon(null);
             dtcr.setOpenIcon(null);
         }
-    } // ClassPanel
+    }
 
     /**
      * Called when the Session has activated. This occurs when the
@@ -146,7 +146,7 @@ public class ClassPanel extends AbstractPanel
         // Update the tree now in case we're connecting to a running VM
         // (in which case there might not be class prepare events).
         SwingUtilities.invokeLater(this);
-    } // activated
+    }
 
     /**
      * Merge the nodes in the existing tree with the new class list.
@@ -206,7 +206,7 @@ public class ClassPanel extends AbstractPanel
             model.addPath(clazz);
             indexNew++;
         }
-    } // consolidateNodes
+    }
 
     /**
      * Called when the Session has deactivated. The debuggee VM is no
@@ -221,7 +221,7 @@ public class ClassPanel extends AbstractPanel
         // The only safe way to make sure the tree is wiped out, and not
         // rebuilt if this method is called during a refresh.
         uitree.setModel(new ClassTreeModel(new BasicNode()));
-    } // deactivated
+    }
 
     /**
      * Enable or disable the automatic refresh feature.
@@ -256,7 +256,7 @@ public class ClassPanel extends AbstractPanel
             vmeman.removeListener(ClassUnloadEvent.class, this);
         }
         autoRefreshing = auto;
-    } // enableAutoRefresh
+    }
 
     /**
      * Invoked when a VM event has occurred.
@@ -271,7 +271,7 @@ public class ClassPanel extends AbstractPanel
             refreshLater();
         }
         return true;
-    } // eventOccurred
+    }
 
     /**
      * Finds the tree node matching the given class name. The name must
@@ -289,7 +289,7 @@ public class ClassPanel extends AbstractPanel
             target = target.getChild(tokenizer.nextToken());
         }
         return target;
-    } // findNode
+    }
 
     /**
      * Converts the TreePath into a String object using the user object
@@ -332,7 +332,7 @@ public class ClassPanel extends AbstractPanel
             name = name.substring(0, cli);
         }
         return name;
-    } // getPathName
+    }
 
     /**
      * Returns a reference to the peer UI component. In many cases this
@@ -343,7 +343,7 @@ public class ClassPanel extends AbstractPanel
      */
     public JComponent getPeer() {
         return uitree;
-    } // getPeer
+    }
 
     /**
      * Returns a reference to the UI component.
@@ -352,7 +352,7 @@ public class ClassPanel extends AbstractPanel
      */
     public JComponent getUI() {
         return uicomp;
-    } // getUI
+    }
 
     /**
      * Returns the last component of the fully-qualified class name.
@@ -362,7 +362,7 @@ public class ClassPanel extends AbstractPanel
      */
     protected static String justTheName(String cname) {
         return cname.substring(cname.lastIndexOf('.') + 1);
-    } // justTheName
+    }
 
     /**
      * Called whenever a node in the tree has been double-clicked on.
@@ -370,24 +370,28 @@ public class ClassPanel extends AbstractPanel
      * @param  tpath  TreePath.
      */
     protected void pathSelected(TreePath tpath) {
-        String pathname = getPathName(tpath);
-        if (pathname == null) {
-            return;
-        }
-
         BasicNode node = (BasicNode) tpath.getLastPathComponent();
-        int line = 0;
+        Location location = null;
+        String pathname = null;
         if (node instanceof MethodNode) {
             MethodNode mnode = (MethodNode) node;
-            line = mnode.getLine();
+            location = mnode.getLocation();
+        } else {
+            pathname = getPathName(tpath);
+            if (pathname == null) {
+                return;
+            }
         }
 
-        // Map the class path to a source file, and try to view it.
         PathManager pathman = (PathManager)
             owningSession.getManager(PathManager.class);
         SourceSource source = null;
         try {
-            source = pathman.mapSource(pathname);
+            if (location == null) {
+                source = pathman.mapSource(pathname);
+            } else {
+                source = pathman.mapSource(location);
+            }
         } catch (IOException ioe) {
             // This case is handled below.
         }
@@ -396,14 +400,14 @@ public class ClassPanel extends AbstractPanel
             owningSession.getUIAdapter().showMessage(
                 UIAdapter.MESSAGE_ERROR,
                 Bundle.getString("couldntMapSrcFile"));
-        } else if (!adapter.showFile(source, line, 0)) {
+        } else if (!adapter.showFile(source, 0, 0)) {
             String msg = MessageFormat.format(
                 Bundle.getString("couldntOpenFileMsg"),
                 new Object[] { source.getName() });
             owningSession.getUIAdapter().showMessage(
                 UIAdapter.MESSAGE_ERROR, msg);
         }
-    } // pathSelected
+    }
 
     /**
      * Invoked when the mouse button has been clicked (pressed and
@@ -419,7 +423,7 @@ public class ClassPanel extends AbstractPanel
                 pathSelected(selPath);
             }
         }
-    } // mouseClicked
+    }
 
     /**
      * Invoked when a mouse button has been pressed on a component.
@@ -533,7 +537,7 @@ public class ClassPanel extends AbstractPanel
 
             currentClassesList = classListNames;
         }
-    } // refresh
+    }
 
     /**
      * Invoked whenever a node in the tree is about to be expanded.
@@ -556,7 +560,7 @@ public class ClassPanel extends AbstractPanel
                 cn.populateMethods();
             }
         }
-    } // treeWillExpand
+    }
 
     /**
      * Invoked whenever a node in the tree is about to be collapsed.
@@ -568,7 +572,7 @@ public class ClassPanel extends AbstractPanel
     public void treeWillCollapse(TreeExpansionEvent event)
         throws ExpandVetoException {
         // We couldn't care less.
-    } // treeWillCollapse
+    }
 
     /**
      * Class ClassTreeModel represents a class tree.
@@ -585,7 +589,7 @@ public class ClassPanel extends AbstractPanel
          */
         public ClassTreeModel(BasicNode root) {
             super(root);
-        } // ClassTreeModel
+        }
 
         /**
          * Add the path of this class, and the class node itself, to the
@@ -617,8 +621,8 @@ public class ClassPanel extends AbstractPanel
                 }
                 node = next;
             }
-        } // addPath
-    } // ClassTreeModel
+        }
+    }
 
     /**
      * Implements the basic tree node behavior.
@@ -632,7 +636,7 @@ public class ClassPanel extends AbstractPanel
          */
         public BasicNode() {
             super();
-        } // BasicNode
+        }
 
         /**
          * Constructs a BasicNode with the given name.
@@ -641,7 +645,7 @@ public class ClassPanel extends AbstractPanel
          */
         public BasicNode(String name) {
             super(name);
-        } // BasicNode
+        }
 
         /**
          * Get the child of this node that has the given name.
@@ -658,7 +662,7 @@ public class ClassPanel extends AbstractPanel
             }
             // didn't find a matching child
             return null;
-        } // getChild
+        }
 
         /**
          * Return the name of this node (the user object, assumed to be
@@ -668,8 +672,8 @@ public class ClassPanel extends AbstractPanel
          */
         public String getName() {
             return (String) getUserObject();
-        } // getName
-    } // BasicNode
+        }
+    }
 
     /**
      * Represents a package node, which contains other package and/or
@@ -686,8 +690,8 @@ public class ClassPanel extends AbstractPanel
          */
         public PackageNode(String name) {
             super(name);
-        } // PackageNode
-    } // PackageNode
+        }
+    }
 
     /**
      * Node in our class tree.
@@ -716,7 +720,7 @@ public class ClassPanel extends AbstractPanel
             } else {
                 ourString = getName();
             }
-        } // ClassNode
+        }
 
         /**
          * Returns this path's class loader reference, if any.
@@ -725,7 +729,7 @@ public class ClassPanel extends AbstractPanel
          */
         public ClassLoaderReference getClassLoader() {
             return classLoader;
-        } // getClassLoader
+        }
 
         /**
          * Returns the fully qualified name of the class represented by
@@ -735,7 +739,7 @@ public class ClassPanel extends AbstractPanel
          */
         public String getFQClassName() {
             return classRef.name();
-        } // getFQClassName
+        }
 
         /**
          * Returns true if this node is a leaf, false otherwise.
@@ -748,7 +752,7 @@ public class ClassPanel extends AbstractPanel
             } else {
                 return false;
             }
-        } // isLeaf
+        }
 
         /**
          * Build out the method nodes for this class.
@@ -756,7 +760,6 @@ public class ClassPanel extends AbstractPanel
         public void populateMethods() {
             List methods = classRef.methods();
             Iterator iter = methods.iterator();
-            StringBuffer buffer = new StringBuffer(128);
             List methodChildren = new ArrayList();
             while (iter.hasNext()) {
                 Method method = (Method) iter.next();
@@ -764,36 +767,7 @@ public class ClassPanel extends AbstractPanel
                     // Skip static initializers, they're not methods.
                     continue;
                 }
-
-                // Get the method name.
-                if (method.isConstructor()) {
-                    String methodName = classRef.name();
-                    methodName = justTheName(methodName);
-                    buffer.append(methodName);
-                } else {
-                    buffer.append(method.name());
-                }
-                buffer.append('(');
-
-                // Get the method arguments.
-                List args = method.argumentTypeNames();
-                for (int ii = 0; ii < args.size(); ii++) {
-                    String anArg = (String) args.get(ii);
-                    buffer.append(justTheName(anArg));
-                    if (ii < (args.size() - 1)) {
-                        buffer.append(", ");
-                    }
-                }
-                buffer.append(')');
-
-                // Get the method line number.
-                Location loc = method.location();
-                int line = 0;
-                if (loc != null) {
-                    line = loc.lineNumber();
-                }
-                methodChildren.add(new MethodNode(buffer.toString(), line));
-                buffer = new StringBuffer(128);
+                methodChildren.add(new MethodNode(method));
             }
 
             // Sort the methods and add them to the class node.
@@ -802,7 +776,7 @@ public class ClassPanel extends AbstractPanel
             while (iter.hasNext()) {
                 add((MethodNode) iter.next());
             }
-        } // populateMethods
+        }
 
         /**
          * Returns the string representation of this.
@@ -811,8 +785,8 @@ public class ClassPanel extends AbstractPanel
          */
         public String toString() {
             return ourString;
-        } // toString
-    } // ClassNode
+        }
+    }
 
     /**
      * Represents a method node.
@@ -820,19 +794,42 @@ public class ClassPanel extends AbstractPanel
      * @author  Nathan Fiedler
      */
     protected class MethodNode extends BasicNode implements Comparable {
-        /** Line number of this method. */
-        private int lineNumber;
+        /** Location of the method. */
+        private Location location;
 
         /**
-         * Constructs a MethodNode with the given name and signature.
+         * Constructs a MethodNode from the given Method instance.
          *
-         * @param  desc  description of this method.
-         * @param  line  line number of this method.
+         * @param  method  method to be displayed.
          */
-        public MethodNode(String desc, int line) {
-            super(desc);
-            lineNumber = line;
-        } // MethodNode
+        public MethodNode(Method method) {
+            super();
+            StringBuffer buffer = new StringBuffer(128);
+            // Get the method name.
+            if (method.isConstructor()) {
+                ReferenceType clazz = method.declaringType();
+                String name = justTheName(clazz.name());
+                buffer.append(name);
+            } else {
+                buffer.append(method.name());
+            }
+            buffer.append('(');
+
+            // Get the method arguments.
+            List args = method.argumentTypeNames();
+            for (int ii = 0; ii < args.size(); ii++) {
+                String arg = (String) args.get(ii);
+                buffer.append(justTheName(arg));
+                if (ii < (args.size() - 1)) {
+                    buffer.append(", ");
+                }
+            }
+            buffer.append(')');
+            setUserObject(buffer.toString());
+
+            // Get the method line number.
+            location = method.location();
+        }
 
         /**
          * Compares this object with the specified object for order.
@@ -848,17 +845,17 @@ public class ClassPanel extends AbstractPanel
         public int compareTo(Object o) {
             MethodNode m = (MethodNode) o;
             return getName().compareTo(m.getName());
-        } // compareTo
+        }
 
         /**
-         * Returns the line number of this method, or zero if not known.
+         * Returns the associated location of this method.
          *
-         * @return  line number, or possibly zero.
+         * @return  location of method.
          */
-        public int getLine() {
-            return lineNumber;
-        } // getLine
-    } // MethodNode
+        public Location getLocation() {
+            return location;
+        }
+    }
 
     /**
      * Custom JTree to show custom tooltip, and set the empty selection
@@ -883,7 +880,7 @@ public class ClassPanel extends AbstractPanel
             // Add the popup menu.
             ClassesPopup popup = new ClassesPopup();
             addMouseListener(popup);
-        } // ClassTree
+        }
 
         /**
          * Returns custom tooltip text for this tree.
@@ -905,8 +902,8 @@ public class ClassPanel extends AbstractPanel
                 }
             }
             return tip;
-        } // getToolTipText
-    } // ClassTree
+        }
+    }
 
     /**
      * Class ClassesPopup is a popup menu that allows the user to clear
@@ -941,7 +938,7 @@ public class ClassPanel extends AbstractPanel
 
             hotswapItem = new JMenuItem("N/A");
             hotswapItem.addActionListener(this);
-        } // ClassesPopup
+        }
 
         /**
          * Invoked when a menu item has been selected.
@@ -1023,7 +1020,7 @@ public class ClassPanel extends AbstractPanel
                         Bundle.getString("Class.hotswap.success"));
                 }
             }
-        } // actionPerformed
+        }
 
         /**
          * Invoked when an item has been selected or deselected by the user.
@@ -1043,7 +1040,7 @@ public class ClassPanel extends AbstractPanel
                 preferences.putBoolean("classes_autoRefresh", selected);
                 enableAutoRefresh(selected);
             }
-        } // itemStateChanged
+        }
 
         /**
          * Set the popup menu items enabled or disabled depending on
@@ -1085,7 +1082,7 @@ public class ClassPanel extends AbstractPanel
             } else {
                 selectedClass = null;
             }
-        } // setMenuItemsForEvent
+        }
 
         /**
          * Show the popup menu.
@@ -1095,6 +1092,6 @@ public class ClassPanel extends AbstractPanel
         protected void showPopup(MouseEvent e) {
             setMenuItemsForEvent(e);
             show(e.getComponent(), e.getX(), e.getY());
-        } // showPopup
-    } // ClassesPopup
-} // ClassPanel
+        }
+    }
+}
