@@ -1,6 +1,6 @@
 /*********************************************************************
  *
- *      Copyright (C) 2000-2002 Nathan Fiedler
+ *      Copyright (C) 2000-2003 Nathan Fiedler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,31 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * PROJECT:     JSwat
- * MODULE:      Panel
- * FILE:        ThreadPanel.java
- *
- * AUTHOR:      Nathan Fiedler
- *
- * REVISION HISTORY:
- *      Name    Date            Description
- *      ----    ----            -----------
- *      nf      07/20/00        Initial version
- *      nf      02/24/01        Fixed bug 58: differentiate fields in
- *                              different object instances
- *      nf      03/17/01        Fixed bug 32: watch local variables
- *      nf      04/20/01        Fixed bugs 55, 96, and other mistakes
- *      nf      05/28/01        Add 'this' support; support feature #73.
- *      nf      08/22/01        Fixed bug 172 and related problems
- *      tr      12/09/01        Clean strings for display, limit length.
- *      nf      03/18/02        Fixed bug 410: use toString()
- *      nf      04/07/02        Implemented 423
- *
- * DESCRIPTION:
- *      Defines the class responsible for displaying a table of the
- *      variable values in the VM.
- *
- * $Id: WatchPanel.java 629 2002-10-26 23:03:26Z nfiedler $
+ * $Id: WatchPanel.java 1084 2003-12-03 08:35:33Z nfiedler $
  *
  ********************************************************************/
 
@@ -51,97 +27,129 @@ import com.bluemarsh.jswat.Session;
 import com.bluemarsh.jswat.FieldNotObjectException;
 import com.bluemarsh.jswat.event.ContextChangeEvent;
 import com.bluemarsh.jswat.event.ContextListener;
+import com.bluemarsh.jswat.event.SessionEvent;
 import com.bluemarsh.jswat.event.VMEventListener;
 import com.bluemarsh.jswat.event.VMEventManager;
-import com.bluemarsh.jswat.util.ClassUtils;
-import com.bluemarsh.jswat.util.FieldAndValue;
-import com.bluemarsh.jswat.util.StringUtils;
-import com.bluemarsh.jswat.util.VariableUtils;
-import com.bluemarsh.util.IntHashtable;
-import com.sun.jdi.*;
-import com.sun.jdi.event.*;
-import com.sun.jdi.request.*;
-import java.awt.event.*;
-import java.util.*;
-import javax.swing.*;
-import javax.swing.event.*;
-import javax.swing.table.*;
+import com.bluemarsh.jswat.ui.SmartPopupMenu;
+import com.bluemarsh.jswat.ui.UIAdapter;
+import com.bluemarsh.jswat.util.VariableValue;
+import com.bluemarsh.jswat.util.Strings;
+import com.bluemarsh.jswat.util.Variables;
+import com.sun.jdi.AbsentInformationException;
+import com.sun.jdi.ClassNotPreparedException;
+import com.sun.jdi.IncompatibleThreadStateException;
+import com.sun.jdi.InvalidStackFrameException;
+import com.sun.jdi.LocalVariable;
+import com.sun.jdi.NativeMethodException;
+import com.sun.jdi.ObjectCollectedException;
+import com.sun.jdi.ObjectReference;
+import com.sun.jdi.StackFrame;
+import com.sun.jdi.ThreadReference;
+import com.sun.jdi.Value;
+import com.sun.jdi.VirtualMachine;
+import com.sun.jdi.VMDisconnectedException;
+import com.sun.jdi.event.Event;
+import com.sun.jdi.event.ModificationWatchpointEvent;
+import com.sun.jdi.request.EventRequest;
+import com.sun.jdi.request.EventRequestManager;
+import com.sun.jdi.request.ModificationWatchpointRequest;
+import com.sun.jdi.request.WatchpointRequest;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
+import java.awt.event.MouseEvent;
+import java.awt.event.MouseListener;
+import java.util.ArrayList;
+import java.util.Iterator;
+import java.util.List;
+import java.util.prefs.Preferences;
+import javax.swing.JComponent;
+import javax.swing.JMenuItem;
+import javax.swing.JScrollPane;
+import javax.swing.JTable;
+import javax.swing.MenuSelectionManager;
+import javax.swing.ToolTipManager;
+import javax.swing.event.TableModelEvent;
+import javax.swing.event.TableModelListener;
+import javax.swing.table.DefaultTableColumnModel;
+import javax.swing.table.TableColumnModel;
+import javax.swing.table.TableModel;
 
 /**
- * Class WatchPanel watches the modification events of variables
- * and displays the changed values in a table.
+ * Class WatchPanel watches the modification events of variables and
+ * displays the changed values in a table.
  *
- * <p>This works by maintaining a blank row in the watch table for the
- * user to add new watchpoints. That is, the user will type the name
- * of a variable to watch into the blank row. To remove a watchpoint,
- * the user must clear the name from that row.</p>
+ * <p>This works by maintaining a blank row in the watch table for the user
+ * to add new watchpoints. That is, the user will type the name of a
+ * variable to watch into the blank row. To remove a watchpoint, the user
+ * must clear the name from that row.</p>
  *
  * @author  Nathan Fiedler
  */
-public class WatchPanel extends JSwatPanel implements ContextListener, TableModelListener, VMEventListener {
-    /** Table displaying the threads. */
-    protected JTable table;
-    /** Our UI component - scrollable panel */
-    protected JScrollPane uicomp;
-    /** Reference to the session. */
-    protected Session owningSession;
-    /** List of the existing watchpoint requests. */
-    protected List watchpointList;
-    // Constants for the table column numbers.
+public class WatchPanel extends AbstractPanel
+    implements ContextListener, TableModelListener, VMEventListener {
+    /** Variable name column number. */
     private static final int NAME_COLUMN = 0;
+    /** Variable value column number. */
     private static final int VALUE_COLUMN = 1;
+    /** Table displaying the threads. */
+    private JTable table;
+    /** Our UI component - scrollable panel */
+    private JScrollPane uicomp;
+    /** List of the existing watchpoint requests. */
+    private List watchpointList;
 
     /**
      * Constructs a WatchPanel with the default table.
      */
     public WatchPanel() {
         watchpointList = new ArrayList();
-	String[] columnNames = new String[2];
-	columnNames[0] = Bundle.getString("Watch.nameColumn");
-	columnNames[1] = Bundle.getString("Watch.valueColumn");
-        boolean[] editableCols = new boolean[2];
-        editableCols[0] = true;
-        editableCols[1] = false;
+
+        // Rebuild the table column model from user preferences.
+        TableColumnModel colmod = new DefaultTableColumnModel();
+        Preferences prefs = Preferences.userRoot().node(
+            "com/bluemarsh/jswat/panel/watch");
+        int[] columnWidths = new int[] { 50, 75 };
+        String[] columnNames = new String[2];
+        columnNames[0] = Bundle.getString("Watch.column.name");
+        columnNames[1] = Bundle.getString("Watch.column.value");
+        columnNames = restoreTable(colmod, prefs, columnWidths, columnNames);
+        // Column names are now in adjusted order.
+
+        boolean[] editableCols = new boolean[] { true, false };
         ViewTableModel model = new ViewTableModel(columnNames, editableCols);
         model.addTableModelListener(this);
-        table = new JTable(model);
-
-        // Set the column widths to some hard-coded values.
-        TableColumn column = table.getColumnModel().getColumn(0);
-        column.setPreferredWidth(50);
 
+        // Create the table and all that goes with it.
+        table = new WatchTable(model, colmod);
+        ToolTipManager.sharedInstance().registerComponent(table);
         uicomp = new JScrollPane(table);
-
         WatchPopup popup = new WatchPopup();
-        table.add(popup);
         table.addMouseListener(popup);
     } // WatchPanel
 
     /**
-     * Called when the Session is about to begin an active debugging
-     * session. That is, JSwat is about to debug a debuggee VM.
-     * Panels are not activated in any particular order.
+     * Called when the Session has activated. This occurs when the
+     * debuggee has launched or has been attached to the debugger.
      *
-     * @param  session  Session being activated.
+     * @param  sevt  session event.
      */
-    public void activate(Session session) {
-        owningSession = session;
+    public void activated(SessionEvent sevt) {
         VMEventManager vmeman = (VMEventManager)
-            session.getManager(VMEventManager.class);
+            owningSession.getManager(VMEventManager.class);
         // Listen to all modification watchpoint events.
         vmeman.addListener(ModificationWatchpointEvent.class, this,
                            VMEventListener.PRIORITY_DEFAULT);
 
         // Add ourselves as a context change listener.
         ContextManager ctxtMgr = (ContextManager)
-            session.getManager(ContextManager.class);
+            owningSession.getManager(ContextManager.class);
         ctxtMgr.addContextListener(this);
 
         // Read the previously set watchpoints from the session properties.
         ViewTableModel model = (ViewTableModel) table.getModel();
         for (int n = 1; true; n++) {
             String key = "watchpoint" + Integer.toString(n);
-            String value = session.getProperty(key);
+            String value = owningSession.getProperty(key);
             if (value == null) {
                 // No watchpoint for n; that marks the end.
                 break;
@@ -155,7 +163,7 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
         // Add a row to the table that will be used to create new
         // watchpoints.
         addBlankRow();
-    } // activate
+    } // activated
 
     /**
      * Adds the blank row to the model, to allow entry of a new
@@ -198,17 +206,16 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
     } // addWatchpoint
 
     /**
-     * Removes all of the present watchpoints.
+     * Called when the Session is about to be closed.
+     *
+     * @param  sevt  session event.
      */
-    protected void clearAllWatches() {
-        if (table.isEditing()) {
-            table.removeEditor();
-        }
-        ViewTableModel model = (ViewTableModel) table.getModel();
-        model.clear();
-        watchpointList.clear();
-        addBlankRow();
-    } // clearAllWatches
+    public void closing(SessionEvent sevt) {
+        Preferences prefs = Preferences.userRoot().node(
+            "com/bluemarsh/jswat/panel/watch");
+        saveTable(table.getColumnModel(), prefs);
+        super.closing(sevt);
+    } // closing
 
     /**
      * Invoked when the current context has changed. The context
@@ -219,26 +226,24 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
      */
     public void contextChanged(ContextChangeEvent cce) {
         resolveWatchpoints();
-        refresh(owningSession);
+        refreshLater();
     } // contextChanged
 
     /**
-     * Called when the Session is about to end an active debugging
-     * session. That is, JSwat is about to terminate the connection
-     * with the debuggee VM.
-     * Panels are not deactivated in any particular order.
+     * Called when the Session has deactivated. The debuggee VM is no
+     * longer connected to the Session.
      *
-     * @param  session  Session being deactivated.
+     * @param  sevt  session event.
      */
-    public void deactivate(Session session) {
+    public void deactivated(SessionEvent sevt) {
         // Remove ourselves as a VM event listener.
         VMEventManager vmeman = (VMEventManager)
-            session.getManager(VMEventManager.class);
+            owningSession.getManager(VMEventManager.class);
         vmeman.removeListener(ModificationWatchpointEvent.class, this);
 
         // Remove ourselves as a context change listener.
         ContextManager ctxtMgr = (ContextManager)
-            session.getManager(ContextManager.class);
+            owningSession.getManager(ContextManager.class);
         ctxtMgr.removeContextListener(this);
 
         if (table.isEditing()) {
@@ -257,18 +262,26 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
         for (int i = 0; i < watchpointList.size(); i++) {
             WatchpointListEntry wle = (WatchpointListEntry)
                 watchpointList.get(i);
-            session.setProperty("watchpoint" + Integer.toString(n),
-                                wle.watchName);
+            owningSession.setProperty("watchpoint" + Integer.toString(n),
+                wle.watchName);
             n++;
         }
 
         // Remove any other watchpoints from the properties.
         Object old;
-        do {
-            old = session.setProperty("watchpoint" + Integer.toString(n),
-                                      null);
-            n++;
-        } while (old != null);
+        while (true) {
+            // By getting the property first, we avoid Windows registry
+            // error code 2 in RegDeleteValue(). JRE bug 4709908.
+            old = owningSession.getProperty("watchpoint"
+                + Integer.toString(n));
+            if (old != null) {
+                owningSession.setProperty("watchpoint"
+                    + Integer.toString(n), null);
+                n++;
+            } else {
+                break;
+            }
+        }
 
         watchpointList.clear();
     } // deactivate
@@ -292,16 +305,10 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
             for (int i = 0; i < watchpointList.size(); i++) {
                 WatchpointListEntry entry = (WatchpointListEntry)
                     watchpointList.get(i);
-                if (entry.isFieldVar() && entry.isResolved()) {
-                    // Watchpoint requests are not associated with a specific
-                    // instance, so need to check that ourselves.
-                    if (entry.watchRequest.equals(mwpr) &&
-                        (((object == null) && (entry.watchObject == null)) ||
-                         ((entry.watchObject != null) &&
-                          (entry.watchObject.equals(object))))) {
-                        row = i;
-                        break;
-                    }
+                if (entry.isFieldVar() && entry.watchRequest != null
+                    && entry.watchRequest.equals(mwpr)) {
+                    row = i;
+                    break;
                 }
             }
 
@@ -311,7 +318,7 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
                 if (value == null) {
                     model.setValueAt("null", row, VALUE_COLUMN);
                 } else {
-                    model.setValueAt(StringUtils.cleanForPrinting(
+                    model.setValueAt(Strings.cleanForPrinting(
                         valueToString(value), 100), row, VALUE_COLUMN);
                 }
             }
@@ -320,9 +327,9 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
     } // eventOccurred
 
     /**
-     * Returns a reference to the peer UI component. In many
-     * cases this is a JList, JTree, or JTable, depending on
-     * the type of data being displayed in the panel.
+     * Returns a reference to the peer UI component. In many cases this
+     * is a JList, JTree, or JTable, depending on the type of data being
+     * displayed in the panel.
      *
      * @return  peer ui component object
      */
@@ -340,16 +347,19 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
     } // getUI
 
     /**
-     * Update the display on the screen. Use the given VM
-     * to fetch the desired data.
+     * Update the display on the screen. Use the given Session to fetch
+     * the desired data. This must be run on the AWT event dispatching
+     * thread.
      *
      * @param  session  Debugging Session object.
      */
     public void refresh(Session session) {
-        ThreadReference thread = session.getCurrentThread();
+        ContextManager ctxtman = (ContextManager)
+            session.getManager(ContextManager.class);
+        ThreadReference thread = ctxtman.getCurrentThread();
         String errorMsg = null;
         if (thread == null) {
-            errorMsg = Bundle.getString("nothreadInParen");
+            errorMsg = Bundle.getString("error.threadNotSet");
         }
 
         try {
@@ -359,7 +369,7 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
                     session.getManager(ContextManager.class);
                 StackFrame frame = thread.frame(ctxtMgr.getCurrentFrame());
                 if (frame == null) {
-                    errorMsg = Bundle.getString("noframeInParen");
+                    errorMsg = Bundle.getString("error.noStackFrame");
                 }
 
                 // Get the visible local variables in this frame.
@@ -370,33 +380,27 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
                 for (int ii = watchpointList.size() - 1; ii >= 0; ii--) {
                     WatchpointListEntry entry = (WatchpointListEntry)
                         watchpointList.get(ii);
-                    if (entry.isLocalVar()) {
-                        LocalVariable lvar = entry.watchLocal;
-                        boolean isVisible = false;
-                        try {
-                            isVisible = lvar.isVisible(frame);
-                        } catch (IllegalArgumentException iae) {
-                            // I don't know how this happens, but it
-                            // means the variable is not visible.
-                        }
-                        if (isVisible) {
+                    if (!entry.isFieldVar()) {
+                        LocalVariable lvar = frame.visibleVariableByName(
+                            entry.watchName);
+                        if (lvar != null) {
                             Value val = frame.getValue(lvar);
                             if (val == null) {
                                 updates[ii] = "null";
                             } else {
-                                // By not being a String, this entry
-                                // will be updated in the second phase.
+                                // By not being a String, this entry will
+                                // be updated in the second phase.
                                 updates[ii] = val;
                             }
                         } else {
-                            updates[ii] =
-                                Bundle.getString("undefinedInParen");
+                            updates[ii] = Bundle.getString(
+                                "Watch.error.invisible");
                         }
                     }
                 }
 
                 // Then update the local variable table entries. This
-                // two-phase approach is necessary because ClassUtils'
+                // two-phase approach is necessary because Classes'
                 // callToString() method invalidates the stack frame.
                 ViewTableModel model = (ViewTableModel) table.getModel();
                 for (int ii = updates.length - 1; ii >= 0; ii--) {
@@ -410,7 +414,7 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
                         valueStr = (String) o;
                     } else {
                         // It must be a Value then.
-                        valueStr = StringUtils.cleanForPrinting(
+                        valueStr = Strings.cleanForPrinting(
                             valueToString((Value) o), 100);
                     }
                     // Set cell in value column of that row to new value.
@@ -420,23 +424,18 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
 
         } catch (AbsentInformationException aie) {
             // Skip the local variables when not available.
-            errorMsg = Bundle.getString("noinfoInParen");
+            errorMsg = Bundle.getString("error.noDebugInfo");
         } catch (IncompatibleThreadStateException itse) {
-            errorMsg = Bundle.getString("threadRunningInParen");
+            errorMsg = Bundle.getString("error.threadRunning");
         } catch (IndexOutOfBoundsException ioobe) {
             // If thread has no frame this exception gets thrown.
-            errorMsg = Bundle.getString("noframeInParen");
+            errorMsg = Bundle.getString("error.noStackFrame");
         } catch (InvalidStackFrameException isfe) {
-            errorMsg = Bundle.getString("invalidframeInParen");
+            errorMsg = Bundle.getString("error.invalidFrame");
         } catch (NativeMethodException nme) {
-            errorMsg = Bundle.getString("nativeInParen");
+            errorMsg = Bundle.getString("error.nativeMethod");
         } catch (VMDisconnectedException vmde) {
-            // Do nothing, just return.
-            return;
-        } catch (Exception e) {
-            // All other exceptions result in an immediate abort.
-            e.printStackTrace();
-            errorMsg = Bundle.getString("errorInParen");
+            // This happens a lot.
         }
 
         if (errorMsg != null) {
@@ -445,7 +444,7 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
             for (int ii = watchpointList.size() - 1; ii >= 0; ii--) {
                 WatchpointListEntry entry = (WatchpointListEntry)
                     watchpointList.get(ii);
-                if (entry.isLocalVar()) {
+                if (!entry.isFieldVar()) {
                     model.setValueAt(errorMsg, ii, VALUE_COLUMN);
                 }
             }
@@ -467,74 +466,80 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
                 owningSession.getManager(ContextManager.class);
             ThreadReference thrd = ctxtman.getCurrentThread();
             if (thrd == null) {
-                model.setValueAt(Bundle.getString("nothreadInParen"),
+                model.setValueAt(Bundle.getString("error.threadNotSet"),
                                  row, VALUE_COLUMN);
                 return;
             }
 
             int frameNum = ctxtman.getCurrentFrame();
-            FieldAndValue fieldValue = VariableUtils.getField
+            VariableValue varValue = Variables.getField
                 (entry.watchName, thrd, frameNum);
-            if (fieldValue.field != null) {
+            if (varValue.field() != null) {
                 // Create a watchpoint request for this field.
                 EventRequestManager erm =
                     owningSession.getVM().eventRequestManager();
                 ModificationWatchpointRequest mwpr =
                     (ModificationWatchpointRequest)
-                    erm.createModificationWatchpointRequest(fieldValue.field);
+                    erm.createModificationWatchpointRequest(
+                        varValue.field());
                 mwpr.setSuspendPolicy(EventRequest.SUSPEND_NONE);
+                VirtualMachine vm = mwpr.virtualMachine();
+                if (vm.canUseInstanceFilters()) {
+                    mwpr.addInstanceFilter(varValue.object());
+                } else {
+                    // Warn the user about lack of filter support.
+                    owningSession.getUIAdapter().showMessage(
+                        UIAdapter.MESSAGE_WARNING,
+                        Bundle.getString("Watch.noInstanceFilters"));
+                }
                 mwpr.enable();
-
-                // Remember both the request and the object we're watching
-                // so it will be refreshed for each modification event.
                 entry.watchRequest = mwpr;
-                entry.watchObject = fieldValue.object;
             } else {
                 // Save the LocalVariable for refreshing later.
-                entry.watchLocal = fieldValue.localVar;
+                entry.watchLocal = varValue.localVariable();
             }
 
             // Show the field value in the value column.
             String valueStr = null;
-            if (fieldValue.value == null) {
+            if (varValue.value() == null) {
                 valueStr = "null";
             } else {
-                valueStr = StringUtils.cleanForPrinting(
-                    valueToString(fieldValue.value), 100);
+                valueStr = Strings.cleanForPrinting(
+                    valueToString(varValue.value()), 100);
             }
             model.setValueAt(valueStr, row, VALUE_COLUMN);
             return;
 
         } catch (AbsentInformationException aie) {
-            errorMsg = Bundle.getString("noinfoInParen");
+            errorMsg = Bundle.getString("error.noDebugInfo");
         } catch (ClassNotPreparedException cnpe) {
-            errorMsg = Bundle.getString("classnotloadedInParen");
+            errorMsg = Bundle.getString("Watch.error.classNotLoaded");
         } catch (FieldNotObjectException fnoe) {
-            errorMsg = Bundle.getString("fieldnotobjectInParen");
+            errorMsg = Bundle.getString("Watch.error.fieldNotObject");
         } catch (IllegalThreadStateException itse) {
-            errorMsg = Bundle.getString("illegalthreadstateInParen");
+            errorMsg = Bundle.getString("error.illegalThreadState");
         } catch (IncompatibleThreadStateException itse) {
-            errorMsg = Bundle.getString("threadRunningInParen");
+            errorMsg = Bundle.getString("error.threadRunning");
         } catch (IndexOutOfBoundsException ioobe) {
             // If thread has no frame this exception gets thrown.
-            errorMsg = Bundle.getString("noframeInParen");
+            errorMsg = Bundle.getString("error.noStackFrame");
         } catch (InvalidStackFrameException isfe) {
-            errorMsg = Bundle.getString("invalidframeInParen");
+            errorMsg = Bundle.getString("error.invalidFrame");
         } catch (NativeMethodException nme) {
-            errorMsg = Bundle.getString("nativeInParen");
+            errorMsg = Bundle.getString("error.nativeMethod");
         } catch (NoSuchFieldException nsfe) {
             // May also indicate that the local variable is not
             // yet visible at this location.
-            errorMsg = Bundle.getString("undefinedInParen");
+            errorMsg = Bundle.getString("Watch.error.invisible");
         } catch (ObjectCollectedException oce) {
-            errorMsg = Bundle.getString("objectcollectedInParen");
+            errorMsg = Bundle.getString("Watch.error.objectCollected");
         }
 
         // If we're here, indicate a problem.
         if (errorMsg != null) {
             model.setValueAt(errorMsg, row, VALUE_COLUMN);
         } else {
-            model.setValueAt(Bundle.getString("errorInParen"),
+            model.setValueAt(Bundle.getString("Watch.error.unknown"),
                              row, VALUE_COLUMN);
         }
     } // resolveWatchpoint
@@ -548,7 +553,7 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
         int row = 0;
         while (iter.hasNext()) {
             WatchpointListEntry entry = (WatchpointListEntry) iter.next();
-            if (!entry.isResolved()) {
+            if (entry.isFieldVar() && entry.watchRequest == null) {
                 resolveWatchpoint(row, entry);
             }
             row++;
@@ -582,7 +587,7 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
                 // See if the user is removing an entry.
                 WatchpointListEntry entry = (WatchpointListEntry)
                     watchpointList.remove(row);
-                if (entry.isFieldVar() && entry.isResolved()) {
+                if (entry.isFieldVar() && entry.watchRequest != null) {
                     entry.watchRequest.disable();
                     erm.deleteEventRequest(entry.watchRequest);
                 }
@@ -601,7 +606,7 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
             // will be overwritten by the addWatchpoint() method.
             WatchpointListEntry entry = (WatchpointListEntry)
                 watchpointList.get(row);
-            if (entry.isFieldVar() && entry.isResolved()) {
+            if (entry.isFieldVar() && entry.watchRequest != null) {
                 entry.watchRequest.disable();
                 erm.deleteEventRequest(entry.watchRequest);
             }
@@ -624,44 +629,75 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
      * frame is in an unknown state and must be retrieved again.</p>
      *
      * @param  value  value to be converted to a string.
-     * @return  String if successful, null if error.
+     * @return  value as String, or error message.
      */
     protected String valueToString(Value value) {
-        if ((value instanceof ObjectReference) &&
-            !(value instanceof StringReference)) {
-            ContextManager ctxtman = (ContextManager)
-                owningSession.getManager(ContextManager.class);
-            // XXX - ought to use an arbitrary thread in debuggee
-            ThreadReference thread = ctxtman.getCurrentThread();
-            if (thread == null) {
-                return null;
-            }
-            String s = ClassUtils.callToString(
-                (ObjectReference) value, thread);
-            if (s == null) {
-                return value.toString();
-            } else {
-                return s;
-            }
-        } else {
-            return value.toString();
+        ContextManager ctxtman = (ContextManager)
+            owningSession.getManager(ContextManager.class);
+        ThreadReference thread = ctxtman.getCurrentThread();
+        if (thread == null) {
+            return Bundle.getString("error.threadNotSet");
+        }
+        try {
+            return Variables.printValue(value, thread, ", ");
+        } catch (Exception e) {
+            String s = Strings.exceptionToString(e);
+            owningSession.getUIAdapter().showMessage(
+                UIAdapter.MESSAGE_ERROR, s);
+            return e.getMessage();
         }
     } // valueToString
 
     /**
-     * Class WatchpointListEntry represents any type of watchpoint the
-     * user has specified. This includes field variables of any object
-     * as well as any local variable.
+     * Custom JTable to show custom tooltip.
+     */
+    protected class WatchTable extends JTable {
+        /** Default tooltip text. */
+        private String defaultTip;
+
+        /**
+         * Constructs a WatchTable with the specified models.
+         *
+         * @param  model   TableModel to use.
+         * @param  colmod  TableColumnModel to use.
+         */
+        public WatchTable(TableModel model, TableColumnModel colmod) {
+            super(model, colmod);
+            defaultTip = Bundle.getString("Watch.tooltip");
+        } // WatchTable
+
+        /**
+         * Returns custom tooltip text for this tree.
+         *
+         * @param  me  Mouse event.
+         * @return  Custom tooltip text.
+         */
+        public String getToolTipText(MouseEvent me) {
+            int row = table.rowAtPoint(me.getPoint());
+            int col = table.columnAtPoint(me.getPoint());
+            ViewTableModel model = (ViewTableModel) table.getModel();
+            String value = (String) model.getValueAt(row, col);
+            if (value == null || value.length() == 0) {
+                return defaultTip;
+            } else {
+                return value;
+            }
+        } // getToolTipText
+    } // WatchTable
+
+    /**
+     * Class WatchpointListEntry represents any type of watchpoint the user
+     * has specified. This includes field variables of any object as well
+     * as any local variable.
      */
     protected class WatchpointListEntry {
         /** Name entered by the user to specify the watchpoint. */
-        public String watchName;
+        private String watchName;
         /** WatchpointRequest, if resolved. */
-        public WatchpointRequest watchRequest;
-        /** Object reference, if a field variable. */
-        public ObjectReference watchObject;
-        /** Local variable, if not a field variable. */
-        public LocalVariable watchLocal;
+        private WatchpointRequest watchRequest;
+        /** Local variable, if not a field variable. Used for comparison
+         * only, as the value can become stale after a hotswap. */
+        private LocalVariable watchLocal;
 
         /**
          * Constructs a WatchpointListEntry object.
@@ -681,46 +717,24 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
             // Can't check the object since it may be a static field.
             return watchLocal == null;
         } // isFieldVar
-
-        /**
-         * Indicates if this watchpoint entry is a local variable or not.
-         *
-         * @return  true if this entry is a local variable.
-         */
-        public boolean isLocalVar() {
-            return watchLocal != null;
-        } // isLocalVar
-
-        /**
-         * Indicates if this watchpoint entry has been resolved.
-         *
-         * @return  true if this entry has been resolved.
-         */
-        public boolean isResolved() {
-            return (watchRequest != null) || isLocalVar();
-        } // isResolved
     } // WatchpointListEntry
 
     /**
      * Class WatchPopup is a popup menu that allows the user to
      * clear all of the entries from the watchpoints list.
      */
-    protected class WatchPopup extends JPopupMenu implements ActionListener, MouseListener {
+    protected class WatchPopup extends SmartPopupMenu
+        implements ActionListener {
         /** The Clear menu item. */
-        protected JMenuItem clearMenuItem;
+        private JMenuItem clearMenuItem;
 
         /**
-         * Constructs an WatchPopup that interacts with the given text
-         * component. The popup can offer pasting as well as clearing
-         * of the text component. By default, the popup will allow
-         * copying the selected text to the clipboard.
-         *
-         * @param  panel  the watch panel.
+         * Constructs an WatchPopup that interacts with the panel.
          */
         WatchPopup() {
-            super(Bundle.getString("WatchPopup.label"));
+            super(Bundle.getString("Watch.menu.label"));
             clearMenuItem = new JMenuItem(
-                Bundle.getString("WatchPopup.clearLabel"));
+                Bundle.getString("Watch.menu.clearLabel"));
             clearMenuItem.addActionListener(this);
             add(clearMenuItem);
         } // WatchPopup
@@ -733,71 +747,23 @@ public class WatchPanel extends JSwatPanel implements ContextListener, TableMode
         public void actionPerformed(ActionEvent event) {
             JMenuItem source = (JMenuItem) event.getSource();
             if (source == clearMenuItem) {
-                clearAllWatches();
+                if (table.isEditing()) {
+                    table.removeEditor();
+                }
+                ViewTableModel model = (ViewTableModel) table.getModel();
+                model.clear();
+                watchpointList.clear();
+                addBlankRow();
             }
         } // actionPerformed
 
         /**
-         * Invoked when the mouse has been clicked on a component.
+         * Show the popup menu.
          *
-         * @param  e  Mouse event.
-         */
-        public void mouseClicked(MouseEvent e) {
-        } // mouseClicked
-
-        /**
-         * Invoked when a mouse button has been pressed on a component.
-         * We use this opportunity to show the popup menu.
-         *
-         * @param  e  Mouse event.
-         */
-        public void mousePressed(MouseEvent e) {
-            // Must check this in both 'pressed' and 'released'.
-            showPopup(e);
-        } // mousePressed
-
-        /**
-         * Invoked when a mouse button has been released on a component.
-         * We use this opportunity to show the popup menu.
-         *
-         * @param  e  Mouse event.
-         */
-        public void mouseReleased(MouseEvent e) {
-            // Must check this in both 'pressed' and 'released'.
-            showPopup(e);
-        } // mouseReleased
-
-        /**
-         * Invoked when the mouse enters a component.
-         *
-         * @param  e  Mouse event.
-         */
-        public void mouseEntered(MouseEvent e) {
-        } // mouseEntered
-
-        /**
-         * Invoked when the mouse exits a component.
-         *
-         * @param  e  Mouse event.
-         */
-        public void mouseExited(MouseEvent e) {
-        } // mouseExited
-
-        /**
-         * Decide whether or not to show the popup menu.
-         *
-         * @param  e  Mouse event.
+         * @param  e  mouse event.
          */
         protected void showPopup(MouseEvent e) {
-            if (e.isPopupTrigger()) {
-                // Show the popup menu.
-                show(e.getComponent(), e.getX(), e.getY());
-            } else {
-                // Process the mouse event normally.
-                MenuSelectionManager.defaultManager().processMouseEvent(e);
-                // Make the menu disappear.
-                MenuSelectionManager.defaultManager().clearSelectedPath();
-            }
+            show(e.getComponent(), e.getX(), e.getY());
         } // showPopup
     } // WatchPopup
 } // WatchPanel
