@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: ConsoleAdapter.java 1159 2004-02-25 23:00:25Z nfiedler $
+ * $Id: ConsoleAdapter.java 1387 2004-11-23 07:28:40Z nfiedler $
  *
  ********************************************************************/
 
@@ -402,7 +402,7 @@ public class ConsoleAdapter extends AbstractAdapter {
      */
     public void updateInputPrompt(String prompt) {
         if (prompt == null) {
-            prompt = "";
+            prompt = "> ";
         }
         inputPrompt = prompt;
     } // updateInputPrompt
@@ -464,7 +464,7 @@ public class ConsoleAdapter extends AbstractAdapter {
                     } catch (InterruptedException ie) {
                         // Ignore it.
                     }
-                    outputStream.printPrompt(inputPrompt + "> ");
+                    outputStream.printPrompt(inputPrompt);
                     String str = inputReader.readLine();
                     // User hitting enter resets this flag.
                     outputStream.needPrompt();
