@@ -28,7 +28,7 @@
  *      nf      05/19/03        Initial version
  *      nf      11/27/03        Moved to util package
  *
- * $Id: Types.java 1087 2003-12-05 01:44:10Z nfiedler $
+ * $Id: Types.java 1456 2005-01-26 09:18:50Z nfiedler $
  *
  ********************************************************************/
 
@@ -458,6 +458,37 @@ public class Types {
         return name;
     } // jniToTypeName
 
+    /**
+     * Creates a mirror of the given object in the given VM.
+     *
+     * @param  o   object to make a mirror of.
+     * @param  vm  virtual machine in which to create mirror.
+     * @return  mirrored value, or original value.
+     */
+    public static Object mirrorOf(Object o, VirtualMachine vm) {
+        if (o instanceof String) {
+            return vm.mirrorOf((String) o);
+        } else if (o instanceof Boolean) {
+            return vm.mirrorOf(((Boolean) o).booleanValue());
+        } else if (o instanceof Character) {
+            return vm.mirrorOf(((Character) o).charValue());
+        } else if (o instanceof Double) {
+            return vm.mirrorOf(((Double) o).doubleValue());
+        } else if (o instanceof Float) {
+            return vm.mirrorOf(((Float) o).floatValue());
+        } else if (o instanceof Integer) {
+            return vm.mirrorOf(((Integer) o).intValue());
+        } else if (o instanceof Long) {
+            return vm.mirrorOf(((Long) o).longValue());
+        } else if (o instanceof Short) {
+            return vm.mirrorOf(((Short) o).shortValue());
+        } else if (o instanceof Byte) {
+            return vm.mirrorOf(((Byte) o).byteValue());
+        } else {
+            return o;
+        }
+    }
+
     /**
      * Converts a class name to a JNI type signature. This simply adds
      * an 'L' prefix, a ';' suffix, and replaces all occurances of '.'
