@@ -1,6 +1,6 @@
 /*********************************************************************
  *
- *      Copyright (C) 2002-2003 Nathan Fiedler
+ *      Copyright (C) 2002-2005 Nathan Fiedler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,18 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * PROJECT:     JSwat
- * MODULE:      View
- * FILE:        BreakpointDrawLayer.java
- *
- * AUTHOR:      Nathan Fiedler
- *
- * REVISION HISTORY:
- *      Name    Date            Description
- *      ----    ----            -----------
- *      nf      01/01/02        Initial version
- *
- * $Id: BreakpointDrawLayer.java 1151 2004-02-25 00:54:39Z nfiedler $
+ * $Id: BreakpointDrawLayer.java 1768 2005-07-03 23:18:52Z nfiedler $
  *
  ********************************************************************/
 
@@ -40,8 +29,11 @@ import com.bluemarsh.jswat.breakpoint.BreakpointListener;
 import com.bluemarsh.jswat.breakpoint.BreakpointManager;
 import com.bluemarsh.jswat.breakpoint.LocatableBreakpoint;
 import com.bluemarsh.jswat.lang.ClassDefinition;
+import com.sun.jdi.AbsentInformationException;
 import com.sun.jdi.ReferenceType;
+import com.sun.jdi.VirtualMachine;
 import java.awt.Color;
+import java.io.File;
 import java.util.Hashtable;
 import java.util.Iterator;
 import java.util.List;
@@ -80,7 +72,7 @@ public class BreakpointDrawLayer extends AbstractGutterDrawLayer
         lineColors = new Hashtable();
         // Default to active in the general case.
         setActive(true);
-    } // BreakpointDrawLayer
+    }
 
     /**
      * Invoked when a breakpoint has been added.
@@ -90,7 +82,7 @@ public class BreakpointDrawLayer extends AbstractGutterDrawLayer
     public void breakpointAdded(BreakpointEvent be) {
         setBreakpoint(be.getBreakpoint());
         textArea.repaintGutter();
-    } // breakpointAdded
+    }
 
     /**
      * Invoked when a breakpoint has been modified.
@@ -101,7 +93,7 @@ public class BreakpointDrawLayer extends AbstractGutterDrawLayer
         // Have to rebuild the entire list from scratch since we can't
         // rely on the breakpoint staying in the same location.
         setBreakpoints((BreakpointManager) be.getSource());
-    } // breakpointModified
+    }
 
     /**
      * Invoked when a breakpoint has been removed.
@@ -121,7 +113,7 @@ public class BreakpointDrawLayer extends AbstractGutterDrawLayer
             }
             textArea.repaintGutter();
         }
-    } // breakpointRemoved
+    }
 
     /**
      * Returns the color that best represents the state of this breakpoint.
@@ -145,7 +137,7 @@ public class BreakpointDrawLayer extends AbstractGutterDrawLayer
         } else {
             return Color.green;
         }
-    } // getBreakpointColor
+    }
 
     /**
      * Gets the priority level of this particular draw layer. Typically
@@ -156,7 +148,7 @@ public class BreakpointDrawLayer extends AbstractGutterDrawLayer
      */
     public int getPriority() {
         return PRIORITY;
-    } // getPriority
+    }
 
     /**
      * Returns true if this draw layer wants to take part in the
@@ -167,7 +159,7 @@ public class BreakpointDrawLayer extends AbstractGutterDrawLayer
     public boolean isActive() {
         // We're active if we have breakpoints.
         return super.isActive() && lineColors.size() > 0;
-    } // isActive
+    }
 
     /**
      * Check if the Breakpoint is in the list of class definitions.
@@ -181,18 +173,19 @@ public class BreakpointDrawLayer extends AbstractGutterDrawLayer
             LocatableBreakpoint lbp = (LocatableBreakpoint) bp;
 
             // Check if the source has the same information.
-            if (lbp.matchesSource(sourceSrc.getPackage(),
-                                  sourceSrc.getName())) {
+            String pkg = sourceSrc.getPackage();
+            String src = sourceSrc.getName();
+            if (lbp.matchesSource(pkg, src)) {
                 return true;
             }
 
-            // Iterate our class definitions to find a match.
-            ReferenceType clazz = lbp.getReferenceType();
-            String cname = "";
-            if (clazz != null) {
-                cname = clazz.name();
-            }
             if (classLines != null) {
+                // Iterate our class definitions to find a match.
+                String cname = "";
+                ReferenceType clazz = lbp.getReferenceType();
+                if (clazz != null) {
+                    cname = clazz.name();
+                }
                 Iterator iter = classLines.iterator();
                 while (iter.hasNext()) {
                     ClassDefinition cd = (ClassDefinition) iter.next();
@@ -206,7 +199,7 @@ public class BreakpointDrawLayer extends AbstractGutterDrawLayer
 
         // Probably does not belong to us anyway.
         return false;
-    } // matches
+    }
 
     /**
      * Adds a line attribute to the source row header, appropriate for
@@ -225,7 +218,7 @@ public class BreakpointDrawLayer extends AbstractGutterDrawLayer
                 setLineColor(line, getBreakpointColor(bp));
             }
         }
-    } // setBreakpoint
+    }
 
     /**
      * Iterate all existing breakpoints and create row header attributes
@@ -240,7 +233,7 @@ public class BreakpointDrawLayer extends AbstractGutterDrawLayer
             setBreakpoint((Breakpoint) iter.next());
         }
         textArea.repaintGutter();
-    } // setBreakpoints
+    }
 
     /**
      * Set the list of class definitions.
@@ -249,7 +242,7 @@ public class BreakpointDrawLayer extends AbstractGutterDrawLayer
      */
     public void setClassDefinitions(List lines) {
         classLines = lines;
-    } // setClassDefinitions
+    }
 
     /**
      * Set the color for the given line.
@@ -266,7 +259,7 @@ public class BreakpointDrawLayer extends AbstractGutterDrawLayer
         } else {
             lineColors.put(new Integer(line), color);
         }
-    } // setLineColor
+    }
 
     /**
      * Update the draw context by setting colors, fonts and possibly
@@ -281,5 +274,5 @@ public class BreakpointDrawLayer extends AbstractGutterDrawLayer
         if (c != null) {
             ctx.setBackColor(c);
         }
-    } // updateContext
-} // BreakpointDrawLayer
+    }
+}
