@@ -1,6 +1,6 @@
 /*********************************************************************
  *
- *      Copyright (C) 1999-2003 Nathan Fiedler
+ *      Copyright (C) 1999-2005 Nathan Fiedler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,47 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * PROJECT:     JSwat
- * MODULE:      View
- * FILE:        SourceView.java
- *
- * AUTHOR:      Nathan Fiedler
- *
- * REVISION HISTORY:
- *      Name    Date            Description
- *      ----    ----            -----------
- *      nf      04/08/99        Initial version
- *      nf      01/28/01        Use Java parser for breakpoint setting
- *      nf      04/21/01        Fixed bug 97, view will scroll after
- *                              view becomes visible
- *      nf      05/04/01        Fixed bug 88, highlight current line
- *      nf      05/28/01        Fixed bug 97, again, using Timer
- *      nf      06/16/01        Changed to use new breakpoints code
- *      he      07/02/01        Added configurable tab size
- *      nf      07/22/01        Fixed bug 131, handle parse errors
- *      nf      08/18/01        Fixed bug 146, show parser errors in Log
- *      nf      08/26/01        Fixed bug 196
- *      nf      08/26/01        Fixed bug 190: making the impossible possible
- *      nf      10/25/01        Fixed bug 249
- *      nf      11/10/01        Fixed bug 293
- *      nf      11/14/01        Fixed bug 313
- *      nf      11/17/01        Fixed bug 318: merged ColorSourcView code
- *      nf      11/25/01        Fixed bug 320
- *      nf      11/28/01        Fixed bug 327
- *      nf      11/29/01        Fixed bug 332
- *      nf      12/04/01        Made colorization 10% faster
- *      tr      12/08/01        Fixed bugs 323, 325
- *      nf      12/15/01        Fixed bug 368
- *      nf      01/06/02        Fixed bug 380 with an ugly hack
- *      nf      04/03/02        Fixed bug 419
- *      nf      04/17/02        Using new preferences facility
- *      nf      04/19/02        Implemented RFE 304
- *      nf      04/25/02        Fixed bug 474
- *      nf      01/30/03        Split out Java-specific code
- *      nf      03/06/03        Fixed bug 728
- *      nf      04/29/03        Fixed bug 766
- *
- * $Id: SourceView.java 1065 2003-11-23 00:13:43Z nfiedler $
+ * $Id: SourceView.java 1768 2005-07-03 23:18:52Z nfiedler $
  *
  ********************************************************************/
 
@@ -96,7 +56,7 @@ import javax.swing.JScrollPane;
  * @author  Nathan Fiedler
  */
 public class SourceView extends AbstractView
-    implements PreferenceChangeListener, SessionListener, ContextListener {
+        implements PreferenceChangeListener, SessionListener, ContextListener {
     /** Our preferences node. */
     protected Preferences preferences;
     /** Popup menu for managing breakpoints. Listens to text component. */
@@ -141,7 +101,7 @@ public class SourceView extends AbstractView
         // Create the breakpoint popup menu gadget for the text component.
         popupMenu = new SourceViewPopup(src);
         textComponent.addMouseListener(popupMenu);
-    } // SourceView
+    }
 
     /**
      * Called when the Session has activated. This occurs when the
@@ -154,7 +114,7 @@ public class SourceView extends AbstractView
         ContextManager cmgr = (ContextManager)
             sevt.getSession().getManager(ContextManager.class);
         cmgr.addContextListener(this);
-    } // activated
+    }
 
     /**
      * Called when the Session is about to be closed.
@@ -170,7 +130,7 @@ public class SourceView extends AbstractView
         viewScroller = null;
         textComponent = null;
         popupMenu = null;
-    } // closing
+    }
 
     /**
      * Invoked when the current context has changed. The context change
@@ -192,7 +152,7 @@ public class SourceView extends AbstractView
             // Clear the current stepping line.
             removeHighlight();
         }
-    } // contextChanged
+    }
 
     /**
      * Called when the Session has deactivated. The debuggee VM is no
@@ -209,7 +169,7 @@ public class SourceView extends AbstractView
         if (logger.isLoggable(Level.INFO)) {
             logger.info("setting current line to 0 in " + viewTitle);
         }
-    } // deactivated
+    }
 
     /**
      * Returns the long version of title of this view. This may be a
@@ -220,7 +180,7 @@ public class SourceView extends AbstractView
      */
     public String getLongTitle() {
         return sourceSrc.getLongName();
-    } // getLongTitle
+    }
 
     /**
      * Returns a reference to the UI component.
@@ -229,7 +189,7 @@ public class SourceView extends AbstractView
      */
     public JComponent getUI() {
         return viewScroller;
-    } // getUI
+    }
 
     /**
      * Check if the Location falls within the displayed source file.
@@ -250,7 +210,7 @@ public class SourceView extends AbstractView
         } catch (AbsentInformationException aie) {
             return false;
         }
-    } // matches
+    }
 
     /**
      * Called after the Session has added this listener to the Session
@@ -263,7 +223,7 @@ public class SourceView extends AbstractView
         BreakpointManager bpman = (BreakpointManager)
             session.getManager(BreakpointManager.class);
         bpman.addBreakListener(breakpointDrawLayer);
-    } // opened
+    }
 
     /**
      * This method gets called when a preference is added, removed or
@@ -283,7 +243,7 @@ public class SourceView extends AbstractView
         if (viewContent != null) {
             setTextContent();
         }
-    } // preferenceChange
+    }
 
     /**
      * Read the input stream text into the text component. The view must
@@ -324,7 +284,7 @@ public class SourceView extends AbstractView
         }
         refreshPost();
         scrollToLine(line);
-    } // refresh
+    }
 
     /**
      * Code to be executed after the refresh() method has completed.
@@ -338,7 +298,7 @@ public class SourceView extends AbstractView
             logger.info("breakpoints and other line attributes set for "
                         + viewTitle);
         }
-    } // refreshPost
+    }
 
     /**
      * Called when the debuggee is about to be resumed.
@@ -346,7 +306,7 @@ public class SourceView extends AbstractView
      * @param  sevt  session event.
      */
     public void resuming(SessionEvent sevt) {
-    } // resuming
+    }
 
     /**
      * Called to update this view's preferences, either when this object
@@ -373,7 +333,7 @@ public class SourceView extends AbstractView
 
         // Set the tab width of the text area.
         tabSize = preferences.getInt("tabWidth", Defaults.VIEW_TAB_WIDTH);
-    } // setPreferences
+    }
 
     /**
      * Set the content of the text component using the text defined
@@ -384,7 +344,7 @@ public class SourceView extends AbstractView
         SourceContent doc = new SourceContent(viewContent);
         // Force a repaint.
         textComponent.setContent(doc);
-    } // setTextContent
+    }
 
     /**
      * Called when the debuggee has been suspended.
@@ -392,5 +352,5 @@ public class SourceView extends AbstractView
      * @param  sevt  session event.
      */
     public void suspended(SessionEvent sevt) {
-    } // suspended
-} // SourceView
+    }
+}
