@@ -1,6 +1,6 @@
 /*********************************************************************
  *
- *      Copyright (C) 2001-2003 Nathan Fiedler
+ *      Copyright (C) 2001-2004 Nathan Fiedler
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -16,27 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * PROJECT:     JSwat
- * MODULE:      View
- * FILE:        SourceViewTextArea.java
- *
- * AUTHOR:      Nathan Fiedler
- *
- * REVISION HISTORY:
- *      Name    Date            Description
- *      ----    ----            -----------
- *      nf      12/08/01        Initial version
- *      nf      01/06/02        Fixed bug 377
- *      nf      02/15/02        Fixed bug 393
- *      nf      02/20/02        Fixed bug 399
- *      nf      03/28/02        Fixed bug 414
- *      nf      04/15/02        Implemented RFE 433
- *      nf      10/20/02        Fixed bug 650
- *      nf      12/28/02        Fixed bug 693
- *      nf      01/26/03        Removed logging calls
- *      nf      11/05/03        Implemented RFE 658
- *
- * $Id: SourceViewTextArea.java 1051 2003-11-17 05:51:45Z nfiedler $
+ * $Id: SourceViewTextArea.java 1312 2004-09-29 05:19:57Z nfiedler $
  *
  ********************************************************************/
 
@@ -50,18 +30,24 @@ import java.awt.FontMetrics;
 import java.awt.Graphics;
 import java.awt.Point;
 import java.awt.Rectangle;
+import java.awt.Toolkit;
+import java.awt.datatransfer.StringSelection;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
+import java.awt.event.KeyEvent;
 import java.awt.event.MouseEvent;
 import java.awt.event.MouseListener;
 import java.awt.event.MouseMotionListener;
 import java.util.ArrayList;
 import java.util.Iterator;
+import java.util.prefs.Preferences;
 import javax.swing.JComponent;
 import javax.swing.JViewport;
+import javax.swing.KeyStroke;
 import javax.swing.Scrollable;
 import javax.swing.SwingConstants;
 import javax.swing.ToolTipManager;
 import javax.swing.text.BadLocationException;
-import java.util.prefs.Preferences;
 
 /**
  * Class SourceViewTextArea is a read-only text component.
@@ -75,7 +61,7 @@ import java.util.prefs.Preferences;
  * @author  David Taylor
  * @author  Masaru Ohba
  */
-public class SourceViewTextArea extends JComponent implements Scrollable {
+public class SourceViewTextArea extends JComponent implements ActionListener, Scrollable {
     /** Color for the gutter background. */
     private static Color gutterColor = Color.lightGray;
     /** Content that contains the text. */
@@ -113,8 +99,29 @@ public class SourceViewTextArea extends JComponent implements Scrollable {
         // This is only half of it; we still need to do the work in the
         // mouseDragged() method.
         setAutoscrolls(true);
+
+        // Enable copy functionality.
+        KeyStroke copy = KeyStroke.getKeyStroke(KeyEvent.VK_C,
+            ActionEvent.CTRL_MASK, false);
+        this.registerKeyboardAction(this, "COPY", copy, JComponent.WHEN_FOCUSED);
     } // SourceViewTextArea
 
+    /**
+     * An action is being performed.
+     *
+     * @param  e  action event.
+     */
+    public void actionPerformed(ActionEvent e) {
+        if (e.getActionCommand().equals("COPY")) {
+            String selText = getSelectedText();
+            if (!selText.equals("")) {
+                StringSelection strsel  = new StringSelection(selText);
+                Toolkit.getDefaultToolkit().getSystemClipboard().setContents(
+                    strsel, strsel);
+            }
+        }
+    } // actionPerformed
+
     /**
      * Adds the given draw layer to this text area.
      *
@@ -413,6 +420,23 @@ public class SourceViewTextArea extends JComponent implements Scrollable {
         return false;
     } // getScrollableTracksViewportHeight
 
+    /**
+     * This function returns the text that is currently selected in the source
+     * viewer. Returns an empty string when there is no selection.
+     *
+     * @return  the selected text, or empty string if no selection.
+     */
+    public String getSelectedText() {
+        if (getSelectionStart() != getSelectionEnd()) {
+            String selText = new String(getContent().getBuffer(),
+                getSelectionStart(), getSelectionEnd() -
+                getSelectionStart());
+            return selText;
+        } else {
+            return "";
+        }
+    } // getSelectedText
+
     /**
      * Returns the selected text's end position. Return 0 if the document
      * is empty, or the value of dot if there is no selection.
