@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  *
- * $Id: JSwatPlugin.java 1246 2004-04-30 03:07:08Z nfiedler $
+ * $Id: JSwatPlugin.java 1477 2005-02-05 07:04:28Z nfiedler $
  *
  ********************************************************************/
 
@@ -38,6 +38,7 @@ import org.gjt.sp.jedit.jEdit;
 import org.gjt.sp.jedit.Buffer;
 import org.gjt.sp.jedit.EBMessage;
 import org.gjt.sp.jedit.EditPane;
+import org.gjt.sp.jedit.EditPlugin;
 import org.gjt.sp.jedit.GUIUtilities;
 import org.gjt.sp.jedit.MiscUtilities;
 import org.gjt.sp.jedit.View;
@@ -85,17 +86,30 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
     /** Prefix for getting properties. */
     private static final String PROPERTY_PREFIX =
         "plugin.jswatplugin.JSwatPlugin.";
+    /** The JSwatPlugin singleton instance */
+    private static JSwatPlugin instance;
     /** The JSwat Session. */
-    private static Session theSession;
+    private Session theSession;
     /** The JEdit UIAdapter. */
-    private static JEditUIAdapter theUIAdapter;
+    private JEditUIAdapter theUIAdapter;
     /** Maps text areas to class definition lists. */
-    private static Map textAreaClassesMap = new HashMap();
+    private Map textAreaClassesMap = new HashMap();
     /** True if the buffer changed and we need to reparse the buffer to
      * determine the class names of the buffer. */
-    private static boolean needReparse = false;
-    /** True if @link{#getDock(View)} has never been called. */
-    private static boolean  firstCallToGetDock = true;
+    private boolean needReparse = false;
+    /** Maps text areas to TextAreaDecorators instances */
+    private Map textAreaDecoratorsMap = new HashMap();
+
+    /**
+     * Get the JSwatPlugin singleton instance.
+     */
+    public static JSwatPlugin getInstance()
+    {
+        if(instance == null)
+            instance = (JSwatPlugin) jEdit.getPlugin(
+                "com.bluemarsh.jswat.plugins.jedit.JSwatPlugin", true);
+        return instance;
+    }
 
     /**
      * Called by jEdit when the plugin is to be started.
@@ -104,37 +118,33 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
         // If the tools.jar file contains the JPDA classes, this method
         // call will save the user from having to deal with it.
         MiscUtilities.isToolsJarAvailable();
+        // Create our singleton session.
         createSession();
+        // Attach our decorators to all open views.
+        View[] views = jEdit.getViews();
+        for (int i = 0; i < views.length; i++) {
+            addTextAreaDecorators(views[i]);
+        }
     } // start
 
     /**
-     * Called by jEdit when it is about to terminate.
+     * Called by jEdit when it is about to terminate, or when the plugin is
+     * unloaded.
      */
     public void stop() {
         if (theSession != null) {
             Main.endSession(theSession);
             theSession = null;
         }
+        // Remove our decorators from all open views.
+        View[] views = jEdit.getViews();
+        for (int i = 0; i < views.length; i++) {
+            removeTextAreaDecorators(views[i]);
+        }
+        // Forget the JSwatPlugin singleton instance
+        instance = null;
     } // stop
 
-    /**
-     * Creates the plugin's menu items.
-     *
-     * @param  menuItems  list of menu items.
-     */
-    public void createMenuItems(Vector menuItems) {
-        menuItems.addElement(GUIUtilities.loadMenu(MENU));
-    } // createMenuItems
-
-    /**
-     * Create the plugin's options pane.
-     *
-     * @param  od  options dialog.
-     */
-    public void createOptionPanes(OptionsDialog od) {
-        od.addOptionPane(new JSwatOptionPane());
-    } // createOptionPanes
-
     /**
      * Called by jEdit to get the dockable component.
      * This method returns a new panel of the JEditUIAdapter.
@@ -142,9 +152,11 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      * @param  view  View instance.
      * @return  the dockable component.
      */
-    public static JComponent getDock(View view) {
-        if (theUIAdapter.getView() == view || firstCallToGetDock) {
-            firstCallToGetDock = false;
+    public JComponent getDock(View view) {
+        if (theUIAdapter.getView() == null) {
+            theUIAdapter.setView(view);
+            return theUIAdapter.getPanel();
+        } else if (theUIAdapter.getView() == view) {
             return theUIAdapter.getPanel();
         } else {
             JPanel panel = new JPanel(new BorderLayout());
@@ -172,7 +184,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      * @throws  ParserException
      *          if file is not valid Java source.
      */
-    static String getClassnameAtLine(JEditTextArea textArea, int lineNo)
+    String getClassnameAtLine(JEditTextArea textArea, int lineNo)
         throws IOException, LexerException, ParserException {
         List classLines = getClassDefinitions(textArea);
         if (classLines == null) {
@@ -193,7 +205,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      * @return  a fully qualified classname, or null if the buffer doesn't
      *   contain any class definition at the specified line.
      */
-    public static String getClassnameAtCursor(View view)
+    public String getClassnameAtCursor(View view)
         throws IOException, LexerException, ParserException
     {
         JEditTextArea textArea = view.getTextArea();
@@ -219,12 +231,15 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      * @throws  ParserException
      *          if file is not valid Java source.
      */
-    static List getClassDefinitions(JEditTextArea textArea)
+    List getClassDefinitions(JEditTextArea textArea)
         throws IOException, LexerException, ParserException {
         List classLines = (List) textAreaClassesMap.get(textArea);
         if (classLines == null || needReparse) {
             // Parse buffer
             Buffer buffer = textArea.getBuffer();
+            if (!buffer.isLoaded()) {
+                return null;
+            }
             String text = buffer.getText(0, buffer.getLength());
             StringReader reader = new StringReader(text);
             JavaParser parser = new JavaParser(reader);
@@ -241,14 +256,14 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      *
      * @return  the session.
      */
-    public static Session getSession() {
+    public Session getSession() {
         return theSession;
     } // getSession
 
     /**
      * Creates the Session instance.
      */
-    private static void createSession() {
+    private void createSession() {
         // Call Main.init() because it might do something useful.
         Main.init();
         // Set the logging properties. By default we log to a file in the
@@ -260,7 +275,8 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
         try {
             manager.readConfiguration(is);
         } catch (IOException e) {
-            Log.log(Log.ERROR, "Error reading JSwat logging properties", e);
+            Log.log(Log.ERROR, this,
+                "Error reading JSwat logging properties: " + e);
         }
         // Set UI adapter class.
         Main.setUIAdapter(JEditUIAdapter.class);
@@ -282,16 +298,12 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
             JEditTextArea textArea = editPane.getTextArea();
 
             if (epu.getWhat() == EditPaneUpdate.CREATED) {
-                BreakpointHighlight breakpointHighlight =
-                    new BreakpointHighlight(textArea);
-                textArea.getGutter().addExtension(
-                    textArea.getGutter().HIGHEST_LAYER,
-                    breakpointHighlight);
-                VariableValueTooltip vvt = new VariableValueTooltip(textArea);
-                textArea.getPainter().addExtension(vvt);
+                addTextAreaDecorators(textArea);
                 needReparse = true;
             } else if (epu.getWhat() == EditPaneUpdate.BUFFER_CHANGED) {
                 needReparse = true;
+            } else if (epu.getWhat() == EditPaneUpdate.DESTROYED) {
+                removeTextAreaDecorators(textArea);
             }
 
         } else if (message instanceof BufferUpdate) {
@@ -308,12 +320,17 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
                     if (theUIAdapter.getView() == null) {
                         theUIAdapter.setView(vu.getView());
                     }
+                    // Attach our decorators to all available text areas.
+                    addTextAreaDecorators(vu.getView());
                 } else if (vu.getWhat() == ViewUpdate.CLOSED) {
                     View closedView = vu.getView();
+                    // Remove our decorators from all available text areas.
+                    removeTextAreaDecorators(closedView);
+                    // If the user closed the view that holds the UIAdapter,
+                    // associate the UIAdapter with another arbitrarily chosen
+                    // view (in case there is one).
+                    View[] views = jEdit.getViews();
                     if (theUIAdapter.getView() == closedView) {
-                        // Associate the UIAdapter with another arbitrarily
-                        // chosen view (in case there is any other open view).
-                        View[] views = jEdit.getViews();
                         for (int i = 0; i < views.length; ++i) {
                             if (views[i] != closedView) {
                                 theUIAdapter.setView(views[i]);
@@ -324,16 +341,6 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
                 }
             }
 
-        } else if (message instanceof EditorExiting) {
-            try {
-                if (theSession != null) {
-                    Main.endSession(theSession);
-                    theSession = null;
-                }
-            } catch (IllegalStateException ise) {
-                // do nothing
-            }
-
         } else if (message instanceof PropertiesChanged) {
             if (theUIAdapter != null) {
                 theUIAdapter.propertiesChanged();
@@ -346,7 +353,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      *
      * @param  view  view on which to display help.
      */
-    public static void helpIndex(View view) {
+    public void helpIndex(View view) {
         JEditUIAdapter adapter = (JEditUIAdapter) getSession().getUIAdapter();
         adapter.showHelp(com.bluemarsh.jswat.Bundle.getResource("helpIndex"));
     } // helpIndex
@@ -357,7 +364,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      *
      * @param  view  associated jEdit view.
      */
-    public static void toggleBreakpoint(View view) {
+    public void toggleBreakpoint(View view) {
         try {
             JEditTextArea textArea = view.getTextArea();
             int line = textArea.getCaretLine() + 1;
@@ -382,8 +389,9 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
                 breakMgr.removeBreakpoint(bp);
             }
         } catch (Exception e) {
-            Log.log(Log.ERROR,
-                "An error occured during execution of a JSwat action:", e);
+            Log.log(Log.ERROR, this,
+                "An error occured during execution of a JSwat action:");
+            Log.log(Log.ERROR, this, e);
             GUIUtilities.error(view, OPTION_PREFIX + "error.invokeAction",
                 new Object[] { e.toString() });
         }
@@ -394,7 +402,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      *
      * @param  view  associated jEdit view.
      */
-    public static void runUntil(View view) {
+    public void runUntil(View view) {
         try {
             if (!getSession().isActive()) {
                 return;
@@ -420,8 +428,9 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
 
             vmResume(view);
         } catch (Exception e) {
-            Log.log(Log.ERROR,
-                "An error occured during execution of a JSwat action:", e);
+            Log.log(Log.ERROR, this,
+                "An error occured during execution of a JSwat action:");
+            Log.log(Log.ERROR, this, e);
             GUIUtilities.error(view, OPTION_PREFIX + "error.invokeAction",
                 new Object[] { e.toString() });
         }
@@ -433,7 +442,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      * @param  view  current view.
      * @return  true to continue, false if user cancelled.
      */
-    public static boolean showOptions(View view) {
+    public boolean showOptions(View view) {
         JSwatOptionPane options = new JSwatOptionPane();
         options._init();
         Icon icon = new ImageIcon(com.bluemarsh.jswat.ui.Bundle
@@ -454,7 +463,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      *
      * @param  view  current view.
      */
-    public static void stepInto(View view) {
+    public void stepInto(View view) {
         invokeJSwatAction(view, "step");
     } // stepInto
 
@@ -463,7 +472,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      *
      * @param  view  current view.
      */
-    public static void stepOut(View view) {
+    public void stepOut(View view) {
         invokeJSwatAction(view, "finish");
     } // stepOut
 
@@ -472,7 +481,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      *
      * @param  view  current view.
      */
-    public static void stepOver(View view) {
+    public void stepOver(View view) {
         invokeJSwatAction(view, "next");
     } // stepOver
 
@@ -482,7 +491,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      *
      * @param  view  the current view.
      */
-    public static void vmStart(View view) {
+    public void vmStart(View view) {
         invokeJSwatAction(view, "vmStart");
     } // vmStart
 
@@ -491,7 +500,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      *
      * @param  view  current view.
      */
-    public static void vmStop(View view) {
+    public void vmStop(View view) {
         invokeJSwatAction(view, "vmStop");
     } // vmStop
 
@@ -500,7 +509,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      *
      * @param  view  current view.
      */
-    public static void vmResume(View view) {
+    public void vmResume(View view) {
         invokeJSwatAction(view, "vmResume");
     } // vmResume
 
@@ -509,7 +518,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      *
      * @param  view  current view.
      */
-    public static void vmSuspend(View view) {
+    public void vmSuspend(View view) {
         invokeJSwatAction(view, "vmSuspend");
     } // vmSuspend
 
@@ -518,7 +527,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      *
      * @param  view  current view.
      */
-    public static void vmAttach(View view) {
+    public void vmAttach(View view) {
         invokeJSwatAction(view, "vmAttach");
     } // vmSuspend
 
@@ -527,7 +536,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      *
      * @param  view  current view.
      */
-    public static void vmDetach(View view) {
+    public void vmDetach(View view) {
         invokeJSwatAction(view, "vmClose");
     } // vmSuspend
 
@@ -537,7 +546,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      *
      * @param  view  associated jEdit view.
      */
-    public static void hotswapClass(View view) {
+    public void hotswapClass(View view) {
         try {
             String classname = getClassnameAtCursor(view);
 
@@ -549,8 +558,9 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
 
             invokeJSwatCommand(view, "hotswap " + classname);
         } catch (Exception e) {
-            Log.log(Log.ERROR,
-                "An error occured during execution of a JSwat action:", e);
+            Log.log(Log.ERROR, this,
+                "An error occured during execution of a JSwat action:");
+            Log.log(Log.ERROR, this, e);
             GUIUtilities.error(view, OPTION_PREFIX + "error.invokeAction",
                 new Object[] { e.toString() });
         }
@@ -564,7 +574,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      * @param  view  associated jEdit view.
      * @param  actionName  the internal name of the JSwat action.
      */
-    private static void invokeJSwatAction(View view, String actionName) {
+    private void invokeJSwatAction(View view, String actionName) {
         try {
             // Check whether the frame of the panel has been registered.
             // Most JSwat actions require a frame associated with the current
@@ -583,9 +593,10 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
             a.actionPerformed(new ActionEvent(view,
                 ActionEvent.ACTION_PERFORMED, actionName));
         } catch (Exception e) {
-            Log.log(Log.ERROR,
+            Log.log(Log.ERROR, this,
                 "An error occured during execution of JSwat action '"
-                + actionName + "': ", e);
+                + actionName + "': ");
+            Log.log(Log.ERROR, this, e);
             GUIUtilities.error(view, OPTION_PREFIX + "error.invokeAction",
                 new Object[] { e.toString() });
         }
@@ -600,7 +611,7 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
      * @param  command  the command, optionally with arguments, separated
      *         by spaces.
      */
-    public static void invokeJSwatCommand(View view, String command) {
+    public void invokeJSwatCommand(View view, String command) {
         try {
             // Show the JSwat panel.
             view.getDockableWindowManager().showDockableWindow("jswatplugin");
@@ -620,11 +631,98 @@ public class JSwatPlugin extends org.gjt.sp.jedit.EBPlugin {
             // Invoke command.
             commandMgr.handleInput(command);
         } catch (Exception e) {
-            Log.log(Log.ERROR,
+            Log.log(Log.ERROR, this,
                 "An error occured during execution of JSwat command '"
-                + command + "':", e);
+                + command + "':");
+            Log.log(Log.ERROR, this, e);
             GUIUtilities.error(view, OPTION_PREFIX + "error.invokeCommand",
                 new Object[] { command, e.toString() });
         }
     } // invokeJSwatCommand
+
+    /**
+     * Helper class remembering the decorators added to jEdit text areas.
+     */
+    private class TextAreaDecorators {
+        BreakpointHighlight bph;
+        VariableValueTooltip vvt;
+
+        public TextAreaDecorators(
+            BreakpointHighlight bph,
+            VariableValueTooltip vvt)
+        {
+            this.bph = bph;
+            this.vvt = vvt;
+        }
+    } // class TextAreaDecorators
+
+    /**
+     * Attach our breakpoint highlighter and variable value tooltip renderer
+     * to all text areas of the given view.
+     *
+     * @param  view  view to attach to.
+     */
+    private void addTextAreaDecorators(View view) {
+        Log.log(Log.DEBUG, this, "adding text area decorators for view...");
+        // Attach to all available text areas.
+        EditPane[] panes = view.getEditPanes();
+        for (int i = 0; i < panes.length; i++) {
+            EditPane pane = panes[i];
+            JEditTextArea textArea = pane.getTextArea();
+            addTextAreaDecorators(textArea);
+        }
+    } // addTextAreaDecorators
+
+    /**
+     * Attach our breakpoint highlighter and variable value tooltip renderer
+     * to the given text area.
+     *
+     * @param  textArea  text area to attach to.
+     */
+    private void addTextAreaDecorators(JEditTextArea textArea) {
+        removeTextAreaDecorators(textArea);
+        Log.log(Log.DEBUG, this, "adding text area decorators for text area...");
+        BreakpointHighlight bph = new BreakpointHighlight(textArea);
+        textArea.getGutter().addExtension(textArea.getGutter().HIGHEST_LAYER, bph);
+        VariableValueTooltip vvt = new VariableValueTooltip(textArea);
+        textArea.getPainter().addExtension(vvt);
+        textAreaDecoratorsMap.put(textArea, new TextAreaDecorators(bph, vvt));
+    } // addTextAreaDecorators
+
+    /**
+     * Remove our breakpoint highlighter and variable value tooltip renderer
+     * from all text areas of the given view.
+     *
+     * @param  view  view to remove from.
+     */
+    private void removeTextAreaDecorators(View view) {
+        Log.log(Log.DEBUG, this, "removing text area decorators from view...");
+        // Remove from all available text areas.
+        EditPane[] panes = view.getEditPanes();
+        for (int i = 0; i < panes.length; i++) {
+            EditPane pane = panes[i];
+            JEditTextArea textArea = pane.getTextArea();
+            removeTextAreaDecorators(textArea);
+        }
+    } // removeTextAreaDecorators
+
+    /**
+     * Remove our breakpoint highlighter and variable value tooltip renderer
+     * from the given text area (in case they are there).
+     *
+     * @param  textArea  text area to remove from.
+     */
+    private void removeTextAreaDecorators(JEditTextArea textArea) {
+        if (textArea != null) {
+            TextAreaDecorators decorators = (TextAreaDecorators)
+                textAreaDecoratorsMap.remove(textArea);
+            if (decorators != null) {
+                Log.log(Log.DEBUG, this,
+                    "removing text area decorators from text area...");
+                textArea.getGutter().removeExtension(decorators.bph);
+                textArea.getPainter().removeExtension(decorators.vvt);
+            }
+        }
+    } // removeTextAreaDecorators
+
 } // JSwatPlugin
