@@ -1,5 +1,5 @@
 // A sample class to test the debugger.
-// $Author: nfiedler $ $Date: 2002-10-13 00:03:33 -0700 (Sun, 13 Oct 2002) $ $Rev: 604 $
+// $Id: overload.java 1113 2004-01-25 22:25:32Z nfiedler $
 
 import java.awt.*;
 import java.awt.event.*;
@@ -30,7 +30,7 @@ public class overload implements ActionListener {
 		}
 	    });
         mainWin.setSize(300, 100);
-        mainWin.show();
+        mainWin.setVisible(true);
     }
 
     public void actionPerformed(ActionEvent e) {
