@@ -1,5 +1,5 @@
 // JSwat Tutorial example program.
-// $Author: nfiedler $ $Date: 2002-10-13 00:03:33 -0700 (Sun, 13 Oct 2002) $ $Rev: 604 $
+// $Id: tutorial.java 1113 2004-01-25 22:25:32Z nfiedler $
 
 import java.awt.*;
 import java.awt.event.*;
@@ -35,6 +35,6 @@ public class tutorial implements ActionListener {
         Button button = new Button("Push me");
         button.addActionListener(me);
         frame.add(button);
-        frame.show();
+        frame.setVisible(true);
     }
 }
